/*
COPYRIGHT STATEMENT

NOTE: This copyright statement applies to all individual files or
modules in which the developers listed below are mentioned.

This software module was originally developed by the Lucent Technologies
Murray Hill, New Jersey, USA. Contributors to the software
include but are not limited to: Eric Petajan and Jialin Zhong.
This software module is an implementation of a part of one or more
MPEG-4 Visual (ISO/IEC 14496-2) tools as specified by MPEG-4 Visual
(ISO/IEC 14496-2). ISO/IEC gives users of the MPEG-4 Visual standard
free license to this software module or modifications thereof for
use in hardware or software products claiming conformance to the
MPEG-4 Visual (ISO/IEC 14496-2) standard. Those intending to use
this software module in hardware or software products are advised
that its use may infringe existing patents. The original developers
of this software and his/her company, the subsequent editors and
their companies, and ISO/IEC have no liability for use of this
software module or modifications thereof. Copyright is not released
for non MPEG-4 Visual standard conforming products. The original
developers retain full right to use the code for its own purpose,
assign or donate the code for non MPEG-4 Visual conforming products.
This copyright notice must be included in all copies or derivative
works.

Copyright (c) 1997

CONTRIBUTION TO THE WG11 PROGRAM MPEG 4 PLAYER
______________________________________________________________________

STANDARDS PROJECT:  MPEG 4 Player
______________________________________________________________________

TITLE:  3D Line-Based Face Animation Module
______________________________________________________________________

AUTHORS:  Eric Petajan, Jialin Zhong

SOURCE:      Lucent Technologies Inc.
 
CONTACT:	Eric Petajan
		Lucent Technologies
		Room 2B231
		600 Mt. Ave.
		Murray Hill, NJ  07974
		Voice:  +1 908-582-3160
		Fax: +1 908-582-6632
		Email:  edp@bell-labs.com
______________________________________________________________________

ABSTRACT:  this module inputs face animation and definition parameters,
and outputs 3D points, lines, and polygons.
*/

#include "wireface.h"

void rotpoint(float *pnt, float *rctr, float anglex, float angley, float anglez)
{
  int i,j,reflect;
  float tpnt[3],rtpnt[3],r[3],sinx,cosx,siny,cosy,sinz,cosz;

  for(i=0;i<3;i++) tpnt[i] = pnt[i] - rctr[i];/*translate to origin*/
  for(i=0;i<3;i++) {
    if(i==0) r[i] = sqrtf(powf(tpnt[1],2.0) + powf(tpnt[2],2.0));
    else {
      if(i==1) r[i] = sqrtf(powf(tpnt[0],2.0) + powf(rtpnt[2],2.0));
      else r[i] = sqrtf(powf(rtpnt[0],2.0) + powf(rtpnt[1],2.0));
    }
    if(i==0) {
      if(r[0] != 0.0) {
	cosx = fcos(anglex);sinx = fsin(anglex);
	rtpnt[1] = (tpnt[1]*cosx) - (tpnt[2]*sinx);
	rtpnt[2] = (tpnt[1]*sinx) + (tpnt[2]*cosx);
	tpnt[1] = rtpnt[1];
	tpnt[2] = rtpnt[2];
      }
      else {rtpnt[1] = 0.0; rtpnt[2] = 0.0;}
    }
    else {
      if(i==1) {
	if(r[1] !=0.0) {
	  cosy = fcos(angley);siny = fsin(angley);
	  rtpnt[2] = (tpnt[2]*cosy) - (tpnt[0]*siny);
	  rtpnt[0] = (tpnt[2]*siny) + (tpnt[0]*cosy);
	  tpnt[0] = rtpnt[0];
	  tpnt[2] = rtpnt[2];
	}
	else {rtpnt[0] = 0.0; rtpnt[2] = 0.0;}
      }
      else {
	if(r[2] !=0.0) {
	  cosz = fcos(anglez);sinz = fsin(anglez);
	  rtpnt[0] = (tpnt[0]*cosz) - (tpnt[1]*sinz);
	  rtpnt[1] = (tpnt[0]*sinz) + (tpnt[1]*cosz);
	}
	else {rtpnt[0] = 0.0; rtpnt[1] = 0.0;}
      }
    }
  }
  for(i=0;i<3;i++) pnt[i] = rtpnt[i] + rctr[i];/*translate back*/
}

void 
applyfaps(int framenum,FDPtype *FDPs,FAPtype *FAPs)
{
   int i, j, k, m;
   float pg[1000][3],irisrad = 0.0,eyerad = 0.0,tmprad, eyectr[3];
   int grpnum,pntnum,fapnum;
   int **isfeatpnt;
   float ***featpts;
   int **fapmask;
   int **faps;
   float **lfaps;

   isfeatpnt = FDPs->isfeatpnt;
   featpts = FDPs->featpts;
   fapmask = FAPs->fapmask;
   faps = FAPs->faps;
   lfaps = FAPs->lfaps;
   /*tongue*/
#if SHOWTONGUE
   glColor3f(.57,.365,.24);
   i = 0;
   grpnum = 6;
   pntnum = 1;
   if(isfeatpnt[grpnum][pntnum]) {
     glBegin(GL_LINE_LOOP);
     pg[i][0] = featpts[grpnum][pntnum][0] - lfaps[framenum][43];
     if(fapmask[framenum][44]) pg[i][1] = featpts[grpnum][pntnum][1] + lfaps[framenum][44];
     else pg[i][1] = featpts[grpnum][pntnum][1] + lfaps[framenum][46];
     pg[i][2] = featpts[grpnum][pntnum][2] + lfaps[framenum][45];
     rotpoint(pg[i],featpts[7][1],lfaps[framenum][48],lfaps[framenum][49],lfaps[framenum][50]);
     glVertex3fv(pg[i++]);
   }
   else if(!framenum) printf("WARNING: missing group %d, point %d\n",grpnum,pntnum);
   pntnum = 3;
   if(isfeatpnt[grpnum][pntnum]) {
     pg[i][0] = featpts[grpnum][2][0] + (fabs(featpts[grpnum][2][0] - featpts[grpnum][3][0])*fcos(lfaps[framenum][47]));
     pg[i][1] = featpts[grpnum][2][1] + lfaps[framenum][46] + (fabs(featpts[grpnum][2][0] - featpts[grpnum][3][0])*fsin(lfaps[framenum][47]));
     pg[i][2] = featpts[grpnum][pntnum][2] + (lfaps[framenum][45]/2.0);
     rotpoint(pg[i],featpts[7][1],lfaps[framenum][48],lfaps[framenum][49],lfaps[framenum][50]);
     glVertex3fv(pg[i++]);
   }
   pntnum = 4;
   if(isfeatpnt[grpnum][pntnum]) {
     pg[i][0] = featpts[grpnum][2][0] - (fabs(featpts[grpnum][2][0] - featpts[grpnum][4][0])*fcos(lfaps[framenum][47]));
     pg[i][1] = featpts[grpnum][2][1] + lfaps[framenum][46] + (fabs(featpts[grpnum][2][0] - featpts[grpnum][4][0])*fsin(lfaps[framenum][47]));
     pg[i][2] = featpts[grpnum][pntnum][2] + (lfaps[framenum][45]/2.0);
     rotpoint(pg[i],featpts[7][1],lfaps[framenum][48],lfaps[framenum][49],lfaps[framenum][50]);
     glVertex3fv(pg[i++]);
   }
   if(isfeatpnt[6][1]) glEnd();
   i = 0;
   pntnum = 1;
   if(isfeatpnt[grpnum][pntnum]) {
     glBegin(GL_LINE_STRIP);
     pg[i][0] = featpts[grpnum][pntnum][0] - lfaps[framenum][43];
     if(fapmask[framenum][44]) pg[i][1] = featpts[grpnum][pntnum][1] + lfaps[framenum][44];
     else pg[i][1] = featpts[grpnum][pntnum][1] + lfaps[framenum][46];
     pg[i][2] = featpts[grpnum][pntnum][2] + lfaps[framenum][45];
     rotpoint(pg[i],featpts[7][1],lfaps[framenum][48],lfaps[framenum][49],lfaps[framenum][50]);
     glVertex3fv(pg[i++]);
   }
   else if(!framenum) printf("WARNING: missing group %d, point %d\n",grpnum,pntnum);
   pntnum = 2;
   if(isfeatpnt[grpnum][pntnum]) {
     pg[i][0] = featpts[grpnum][pntnum][0];
     pg[i][1] = featpts[grpnum][pntnum][1] + lfaps[framenum][46];
     pg[i][2] = featpts[grpnum][pntnum][2] + (lfaps[framenum][45]/2.0);
     rotpoint(pg[i],featpts[7][1],lfaps[framenum][48],lfaps[framenum][49],lfaps[framenum][50]);
     glVertex3fv(pg[i++]);
   }
   if(isfeatpnt[6][1]) glEnd();
#endif
   /*teeth*/
#if SHOWTEETH
   glColor3f(1.0,1.0,1.0);
   i = 0;
   grpnum = 9;
   pntnum = 8;
   if(isfeatpnt[grpnum][pntnum]) {
     glBegin(GL_LINE_STRIP);
     pg[i][0] = featpts[grpnum][pntnum][0];
     pg[i][1] = featpts[grpnum][pntnum][1];
     pg[i][2] = featpts[grpnum][pntnum][2];
     rotpoint(pg[i],featpts[7][1],lfaps[framenum][48],lfaps[framenum][49],lfaps[framenum][50]);
     glVertex3fv(pg[i++]);
   }
   else if(!framenum) printf("WARNING: missing group %d, point %d\n",grpnum,pntnum);
   pntnum = 10;
   if(isfeatpnt[grpnum][pntnum]) {
     pg[i][0] = featpts[grpnum][pntnum][0];
     pg[i][1] = featpts[grpnum][pntnum][1];
     pg[i][2] = featpts[grpnum][pntnum][2];
     rotpoint(pg[i],featpts[7][1],lfaps[framenum][48],lfaps[framenum][49],lfaps[framenum][50]);
     glVertex3fv(pg[i++]);
   }
   if(isfeatpnt[9][8]) glEnd();
   pntnum = 9;
   if(isfeatpnt[grpnum][pntnum]) {
     glBegin(GL_LINE_STRIP);
     pg[i][0] = featpts[grpnum][pntnum][0] - lfaps[framenum][15];
     pg[i][1] = featpts[grpnum][pntnum][1] - lfaps[framenum][3];
     pg[i][2] = featpts[grpnum][pntnum][2] + lfaps[framenum][14];
     rotpoint(pg[i],featpts[7][1],lfaps[framenum][48],lfaps[framenum][49],lfaps[framenum][50]);
     glVertex3fv(pg[i++]);
   }
   else if(!framenum) printf("WARNING: missing group %d, point %d\n",grpnum,pntnum);
   pntnum = 11;
   if(isfeatpnt[grpnum][pntnum]) {
     pg[i][0] = featpts[grpnum][pntnum][0] - lfaps[framenum][15];
     pg[i][1] = featpts[grpnum][pntnum][1] - lfaps[framenum][3];
     pg[i][2] = featpts[grpnum][pntnum][2] + lfaps[framenum][14];
     rotpoint(pg[i],featpts[7][1],lfaps[framenum][48],lfaps[framenum][49],lfaps[framenum][50]);
     glVertex3fv(pg[i++]);
   }
   if(isfeatpnt[9][9]) glEnd();
#endif
   glColor3f(.52,.21,.16);
   /*inner lip contour*/
   i = 0;
   grpnum = 2;
   pntnum = 2;
   if(isfeatpnt[grpnum][pntnum]) {
     glBegin(GL_LINE_LOOP);
     pg[i][0] = featpts[grpnum][pntnum][0];
     pg[i][1] = featpts[grpnum][pntnum][1] - lfaps[framenum][4];
     pg[i][2] = featpts[grpnum][pntnum][2] + lfaps[framenum][17];
     rotpoint(pg[i],featpts[7][1],lfaps[framenum][48],lfaps[framenum][49],lfaps[framenum][50]);
     glVertex3fv(pg[i++]);
   }
   else if(!framenum) printf("WARNING: missing group %d, point %d\n",grpnum,pntnum);
   pntnum = 6;
   if(isfeatpnt[grpnum][pntnum]) {
     pg[i][0] = featpts[grpnum][pntnum][0] + (lfaps[framenum][6]/2.0);
     pg[i][1] = featpts[grpnum][pntnum][1] - lfaps[framenum][8];
     pg[i][2] = featpts[grpnum][pntnum][2] + (lfaps[framenum][17]/1.0);
     rotpoint(pg[i],featpts[7][1],lfaps[framenum][48],lfaps[framenum][49],lfaps[framenum][50]);
     glVertex3fv(pg[i++]);
   }
   pntnum = 4;
   if(isfeatpnt[grpnum][pntnum]) {
     pg[i][0] = featpts[grpnum][pntnum][0]+ lfaps[framenum][6];
     pg[i][1] = featpts[grpnum][pntnum][1]+ lfaps[framenum][12];
     pg[i][2] = featpts[grpnum][pntnum][2] + lfaps[framenum][17]/1.0;
     rotpoint(pg[i],featpts[7][1],lfaps[framenum][48],lfaps[framenum][49],lfaps[framenum][50]);
     glVertex3fv(pg[i++]);
   }
   pntnum = 8;
   if(isfeatpnt[grpnum][pntnum]) {
     pg[i][0] = featpts[grpnum][pntnum][0] + (lfaps[framenum][6]/2.0);
     pg[i][1] = featpts[grpnum][pntnum][1] + lfaps[framenum][10];
     pg[i][2] = featpts[grpnum][pntnum][2] + (lfaps[framenum][16]/1.0);
     rotpoint(pg[i],featpts[7][1],lfaps[framenum][48],lfaps[framenum][49],lfaps[framenum][50]);
     glVertex3fv(pg[i++]);
   }
   pntnum = 3;
   if(isfeatpnt[grpnum][pntnum]) {
     pg[i][0] = featpts[grpnum][pntnum][0];
     pg[i][1] = featpts[grpnum][pntnum][1] + lfaps[framenum][5];
     pg[i][2] = featpts[grpnum][pntnum][2] + lfaps[framenum][16];
     rotpoint(pg[i],featpts[7][1],lfaps[framenum][48],lfaps[framenum][49],lfaps[framenum][50]);
     glVertex3fv(pg[i++]);
   }
   pntnum = 9;
   if(isfeatpnt[grpnum][pntnum]) {
     pg[i][0] = featpts[grpnum][pntnum][0] - (lfaps[framenum][7]/2.0);
     pg[i][1] = featpts[grpnum][pntnum][1] + lfaps[framenum][11];
     pg[i][2] = featpts[grpnum][pntnum][2] + (lfaps[framenum][16]/1.0);
     rotpoint(pg[i],featpts[7][1],lfaps[framenum][48],lfaps[framenum][49],lfaps[framenum][50]);
     glVertex3fv(pg[i++]);
   }
   pntnum = 5;
   if(isfeatpnt[grpnum][pntnum]) {
     pg[i][0] = featpts[grpnum][pntnum][0] - lfaps[framenum][7];
     pg[i][1] = featpts[grpnum][pntnum][1] + lfaps[framenum][13];
     pg[i][2] = featpts[grpnum][pntnum][2] + lfaps[framenum][17]/1.0;
     rotpoint(pg[i],featpts[7][1],lfaps[framenum][48],lfaps[framenum][49],lfaps[framenum][50]);
     glVertex3fv(pg[i++]);
   }
   pntnum = 7;
   if(isfeatpnt[grpnum][pntnum]) {
     pg[i][0] = featpts[grpnum][pntnum][0] - (lfaps[framenum][7]/2.0);
     pg[i][1] = featpts[grpnum][pntnum][1] - lfaps[framenum][9];
     pg[i][2] = featpts[grpnum][pntnum][2] + (lfaps[framenum][17]/1.0);
     rotpoint(pg[i],featpts[7][1],lfaps[framenum][48],lfaps[framenum][49],lfaps[framenum][50]);
     glVertex3fv(pg[i++]);
   }
   if(isfeatpnt[2][2]) glEnd();
   /*outer lip contour*/
   i = 0;
   grpnum = 8;
   pntnum = 1;
   if(isfeatpnt[grpnum][pntnum]) {
     glBegin(GL_LINE_LOOP);
     pg[i][0] = featpts[grpnum][pntnum][0];
     if(fapmask[framenum][51]) pg[i][1] = featpts[grpnum][pntnum][1] - lfaps[framenum][51];
     else pg[i][1] = featpts[grpnum][pntnum][1] - lfaps[framenum][4];
     pg[i][2] = featpts[grpnum][pntnum][2] + lfaps[framenum][17];
     rotpoint(pg[i],featpts[7][1],lfaps[framenum][48],lfaps[framenum][49],lfaps[framenum][50]);
     glVertex3fv(pg[i++]);
   }
   else if(!framenum) printf("WARNING: missing group %d, point %d\n",grpnum,pntnum);
   pntnum = 10;
   if(isfeatpnt[grpnum][pntnum]) {
     if(fapmask[framenum][53]) pg[i][0] = featpts[grpnum][pntnum][0] + (lfaps[framenum][53]*(fabs(featpts[grpnum][10][0] - featpts[grpnum][1][0])/fabs(featpts[grpnum][3][0] - featpts[grpnum][1][0])));
     else pg[i][0] = featpts[grpnum][pntnum][0] + (lfaps[framenum][6]*(fabs(featpts[grpnum][10][0] - featpts[grpnum][1][0])/fabs(featpts[grpnum][3][0] - featpts[grpnum][1][0])));
     if(fapmask[framenum][51]) pg[i][1] = featpts[grpnum][pntnum][1] - lfaps[framenum][51];
     else pg[i][1] = featpts[grpnum][pntnum][1] - lfaps[framenum][4];
     pg[i][2] = featpts[grpnum][pntnum][2] + lfaps[framenum][17];
     rotpoint(pg[i],featpts[7][1],lfaps[framenum][48],lfaps[framenum][49],lfaps[framenum][50]);
     glVertex3fv(pg[i++]);
   }
   pntnum = 5;
   if(isfeatpnt[grpnum][pntnum]) {
     if(fapmask[framenum][53] && fapmask[framenum][55]) {
       pg[i][0] = featpts[grpnum][pntnum][0] + (lfaps[framenum][53]/2.0);
       pg[i][1] = featpts[grpnum][pntnum][1] - lfaps[framenum][55];
     }
     else {
       pg[i][0] = featpts[grpnum][pntnum][0] + (lfaps[framenum][6]/2.0);
       pg[i][1] = featpts[grpnum][pntnum][1] - lfaps[framenum][8];
     }
     pg[i][2] = featpts[grpnum][pntnum][2] + (lfaps[framenum][17]/1.0);
     rotpoint(pg[i],featpts[7][1],lfaps[framenum][48],lfaps[framenum][49],lfaps[framenum][50]);
     glVertex3fv(pg[i++]);
   }
   pntnum = 3;
   if(isfeatpnt[grpnum][pntnum]) {
     if(fapmask[framenum][53]) pg[i][0] = featpts[grpnum][pntnum][0]+ lfaps[framenum][53];
     else pg[i][0] = featpts[grpnum][pntnum][0]+ lfaps[framenum][6];
     if(fapmask[framenum][59]) pg[i][1] = featpts[grpnum][pntnum][1]+ lfaps[framenum][59];
     else pg[i][1] = featpts[grpnum][pntnum][1]+ lfaps[framenum][12];
     pg[i][2] = featpts[grpnum][pntnum][2] + (lfaps[framenum][17]/1.0);
     rotpoint(pg[i],featpts[7][1],lfaps[framenum][48],lfaps[framenum][49],lfaps[framenum][50]);
     glVertex3fv(pg[i++]);
   }
   pntnum = 7;
   if(isfeatpnt[grpnum][pntnum]) {
     if(fapmask[framenum][53]) pg[i][0] = featpts[grpnum][pntnum][0] + (lfaps[framenum][53]/2.0);
     else pg[i][0] = featpts[grpnum][pntnum][0] + (lfaps[framenum][6]/2.0);
     if(fapmask[framenum][57]) pg[i][1] = featpts[grpnum][pntnum][1] + lfaps[framenum][57];
     else pg[i][1] = featpts[grpnum][pntnum][1] + lfaps[framenum][10];
     pg[i][2] = featpts[grpnum][pntnum][2] + (lfaps[framenum][16]/1.0);
     rotpoint(pg[i],featpts[7][1],lfaps[framenum][48],lfaps[framenum][49],lfaps[framenum][50]);
     glVertex3fv(pg[i++]);
   }
   pntnum = 2;
   if(isfeatpnt[grpnum][pntnum]) {
     pg[i][0] = featpts[grpnum][pntnum][0];
     if(fapmask[framenum][52]) pg[i][1] = featpts[grpnum][pntnum][1] + lfaps[framenum][52];
     else pg[i][1] = featpts[grpnum][pntnum][1] + lfaps[framenum][5];
     pg[i][2] = featpts[grpnum][pntnum][2] + lfaps[framenum][16];
     rotpoint(pg[i],featpts[7][1],lfaps[framenum][48],lfaps[framenum][49],lfaps[framenum][50]);
     glVertex3fv(pg[i++]);
   }
   pntnum = 8;
   if(isfeatpnt[grpnum][pntnum]) {
     if(fapmask[framenum][54]) pg[i][0] = featpts[grpnum][pntnum][0] - (lfaps[framenum][54]/2.0);
     else pg[i][0] = featpts[grpnum][pntnum][0] - (lfaps[framenum][7]/2.0);
     if(fapmask[framenum][58]) pg[i][1] = featpts[grpnum][pntnum][1] + lfaps[framenum][58];
     else pg[i][1] = featpts[grpnum][pntnum][1] + lfaps[framenum][11];
     pg[i][2] = featpts[grpnum][pntnum][2] + (lfaps[framenum][16]/1.0);
     rotpoint(pg[i],featpts[7][1],lfaps[framenum][48],lfaps[framenum][49],lfaps[framenum][50]);
     glVertex3fv(pg[i++]);
   }
   pntnum = 4;
   if(isfeatpnt[grpnum][pntnum]) {
     if(fapmask[framenum][54]) pg[i][0] = featpts[grpnum][pntnum][0] - lfaps[framenum][54];
     else pg[i][0] = featpts[grpnum][pntnum][0] - lfaps[framenum][7];
     if(fapmask[framenum][60]) pg[i][1] = featpts[grpnum][pntnum][1] + lfaps[framenum][60];
     else pg[i][1] = featpts[grpnum][pntnum][1] + lfaps[framenum][13];
     pg[i][2] = featpts[grpnum][pntnum][2] + (lfaps[framenum][16]/1.0);
     rotpoint(pg[i],featpts[7][1],lfaps[framenum][48],lfaps[framenum][49],lfaps[framenum][50]);
     glVertex3fv(pg[i++]);
   }
   pntnum = 6;
   if(isfeatpnt[grpnum][pntnum]) {
     if(fapmask[framenum][54]) pg[i][0] = featpts[grpnum][pntnum][0] - (lfaps[framenum][54]/2.0);
     else pg[i][0] = featpts[grpnum][pntnum][0] - (lfaps[framenum][7]/2.0);
     if(fapmask[framenum][56]) pg[i][1] = featpts[grpnum][pntnum][1] - lfaps[framenum][56];
     else pg[i][1] = featpts[grpnum][pntnum][1] - lfaps[framenum][9];
     pg[i][2] = featpts[grpnum][pntnum][2] + (lfaps[framenum][17]/1.0);
     rotpoint(pg[i],featpts[7][1],lfaps[framenum][48],lfaps[framenum][49],lfaps[framenum][50]);
     glVertex3fv(pg[i++]);
   }
   pntnum = 9;
   if(isfeatpnt[grpnum][pntnum]) {
     if(fapmask[framenum][54]) pg[i][0] = featpts[grpnum][pntnum][0] - (lfaps[framenum][54]*(fabs(featpts[grpnum][9][0] - featpts[grpnum][1][0])/fabs(featpts[grpnum][4][0] - featpts[grpnum][1][0])));
     else pg[i][0] = featpts[grpnum][pntnum][0] - (lfaps[framenum][7]*(fabs(featpts[grpnum][9][0] - featpts[grpnum][1][0])/fabs(featpts[grpnum][4][0] - featpts[grpnum][1][0])));
     if(fapmask[framenum][56]) pg[i][1] = featpts[grpnum][pntnum][1] - lfaps[framenum][56];
     else pg[i][1] = featpts[grpnum][pntnum][1] - lfaps[framenum][4];
     pg[i][2] = featpts[grpnum][pntnum][2] + lfaps[framenum][17];
     rotpoint(pg[i],featpts[7][1],lfaps[framenum][48],lfaps[framenum][49],lfaps[framenum][50]);
     glVertex3fv(pg[i++]);
   }
   if(isfeatpnt[8][1]) glEnd();
   /*jaw*/
   glColor3f(.57,.365,.24);
   i = 0;
   grpnum = 10;
   pntnum = 7;
   if(isfeatpnt[grpnum][pntnum]) {
     glBegin(GL_LINE_STRIP);
     pg[i][0] = featpts[grpnum][pntnum][0];
     pg[i][1] = featpts[grpnum][pntnum][1];
     pg[i][2] = featpts[grpnum][pntnum][2];
     rotpoint(pg[i],featpts[7][1],lfaps[framenum][48],lfaps[framenum][49],lfaps[framenum][50]);
     glVertex3fv(pg[i++]);
   }
   else if(!framenum) printf("WARNING: missing group %d, point %d\n",grpnum,pntnum);
   grpnum = 2;
   pntnum = 13;
   if(isfeatpnt[grpnum][pntnum]) {
     pg[i][0] = featpts[grpnum][pntnum][0] - lfaps[framenum][15];
     pg[i][1] = featpts[grpnum][pntnum][1];
     pg[i][2] = featpts[grpnum][pntnum][2] + lfaps[framenum][14];
     rotpoint(pg[i],featpts[7][1],lfaps[framenum][48],lfaps[framenum][49],lfaps[framenum][50]);
     glVertex3fv(pg[i++]);
   }
   pntnum = 11;
   if(isfeatpnt[grpnum][pntnum]) {
     pg[i][0] = featpts[grpnum][pntnum][0] - lfaps[framenum][15];
     pg[i][1] = featpts[grpnum][pntnum][1] - lfaps[framenum][3];
     pg[i][2] = featpts[grpnum][pntnum][2] + lfaps[framenum][14];
     rotpoint(pg[i],featpts[7][1],lfaps[framenum][48],lfaps[framenum][49],lfaps[framenum][50]);
     glVertex3fv(pg[i++]);
   }
   pntnum = 1;
   if(isfeatpnt[grpnum][pntnum]) {
     pg[i][0] = featpts[grpnum][pntnum][0] - lfaps[framenum][15];
     pg[i][1] = featpts[grpnum][pntnum][1] - lfaps[framenum][3];
     pg[i][2] = featpts[grpnum][pntnum][2] + lfaps[framenum][14];
     rotpoint(pg[i],featpts[7][1],lfaps[framenum][48],lfaps[framenum][49],lfaps[framenum][50]);
     glVertex3fv(pg[i++]);
   }
   pntnum = 12;
   if(isfeatpnt[grpnum][pntnum]) {
     pg[i][0] = featpts[grpnum][pntnum][0] - lfaps[framenum][15];
     pg[i][1] = featpts[grpnum][pntnum][1] - lfaps[framenum][3];
     pg[i][2] = featpts[grpnum][pntnum][2] + lfaps[framenum][14];
     rotpoint(pg[i],featpts[7][1],lfaps[framenum][48],lfaps[framenum][49],lfaps[framenum][50]);
     glVertex3fv(pg[i++]);
   }
   pntnum = 14;
   if(isfeatpnt[grpnum][pntnum]) {
     pg[i][0] = featpts[grpnum][pntnum][0] - lfaps[framenum][15];
     pg[i][1] = featpts[grpnum][pntnum][1];
     pg[i][2] = featpts[grpnum][pntnum][2] + lfaps[framenum][14];
     rotpoint(pg[i],featpts[7][1],lfaps[framenum][48],lfaps[framenum][49],lfaps[framenum][50]);
     glVertex3fv(pg[i++]);
   }
   grpnum = 10;
   pntnum = 8;
   if(isfeatpnt[grpnum][pntnum]) {
     pg[i][0] = featpts[grpnum][pntnum][0];
     pg[i][1] = featpts[grpnum][pntnum][1];
     pg[i][2] = featpts[grpnum][pntnum][2];
     rotpoint(pg[i],featpts[7][1],lfaps[framenum][48],lfaps[framenum][49],lfaps[framenum][50]);
     glVertex3fv(pg[i++]);
   }
   if(isfeatpnt[10][7]) glEnd();
   /*chin boss*/
   i = 0;
   grpnum = 2;
   pntnum = 12;
   if(isfeatpnt[grpnum][pntnum]) {
     glBegin(GL_LINE_STRIP);
     pg[i][0] = featpts[grpnum][pntnum][0] - lfaps[framenum][15];
     pg[i][1] = featpts[grpnum][pntnum][1] - lfaps[framenum][3];
     pg[i][2] = featpts[grpnum][pntnum][2] + lfaps[framenum][14];
     rotpoint(pg[i],featpts[7][1],lfaps[framenum][48],lfaps[framenum][49],lfaps[framenum][50]);
     glVertex3fv(pg[i++]);
   }
   else if(!framenum) printf("WARNING: missing group %d, point %d\n",grpnum,pntnum);
   pntnum = 10;
   if(isfeatpnt[grpnum][pntnum]) {
     pg[i][0] = featpts[grpnum][pntnum][0] - lfaps[framenum][15];
     pg[i][1] = featpts[grpnum][pntnum][1] - lfaps[framenum][3] + lfaps[framenum][18];
     pg[i][2] = featpts[grpnum][pntnum][2] + lfaps[framenum][14];
     rotpoint(pg[i],featpts[7][1],lfaps[framenum][48],lfaps[framenum][49],lfaps[framenum][50]);
     glVertex3fv(pg[i++]);
   }
   pntnum = 11;
   if(isfeatpnt[grpnum][pntnum]) {
     pg[i][0] = featpts[grpnum][pntnum][0] - lfaps[framenum][15];
     pg[i][1] = featpts[grpnum][pntnum][1] - lfaps[framenum][3];
     pg[i][2] = featpts[grpnum][pntnum][2] + lfaps[framenum][14];
     rotpoint(pg[i],featpts[7][1],lfaps[framenum][48],lfaps[framenum][49],lfaps[framenum][50]);
     glVertex3fv(pg[i++]);
   }
   if(isfeatpnt[2][12]) glEnd();
   /*draw iris with lines*/
   /*left iris and pupil*/
   glColor3f(0,0,.78);
   if(isfeatpnt[3][11] && isfeatpnt[3][7]) {/* assume that eyeball diameter is same as eyelid width*/
     eyerad = fabs(featpts[3][7][0] - featpts[3][11][0])/2.0;
   }
   if(isfeatpnt[3][1] && isfeatpnt[3][3]) {
     irisrad = fabs(featpts[3][1][1] - featpts[3][3][1])/2.0;
     glBegin(GL_LINE_LOOP);
   }
   else if(!framenum) printf("WARNING: missing group 3, points 1 and 3\n");
   for(i=0;i<3;i++) eyectr[i] = featpts[3][5][i];
   eyectr[2] -= eyerad;
   for(i=0;i<IRISLINES;i++) {
     pg[2*i][0] = (irisrad*fcos((M_PI_2*i)/IRISLINES)) + featpts[3][5][0];
     pg[2*i][1] = (irisrad*fsin((M_PI_2*i)/IRISLINES)) + featpts[3][5][1];
     pg[2*i][2] = featpts[3][5][2] + lfaps[framenum][27];
     rotpoint(pg[2*i],eyectr,lfaps[framenum][25],lfaps[framenum][23],0.0);
     rotpoint(pg[2*i],featpts[7][1],lfaps[framenum][48],lfaps[framenum][49],lfaps[framenum][50]);
     glVertex3fv(pg[2*i]);
     pg[(2*i)+1][0] = ((lfaps[framenum][29]/2.0)*fcos((M_PI_2*i)/IRISLINES)) + featpts[3][5][0];
     pg[(2*i)+1][1] = ((lfaps[framenum][29]/2.0)*fsin((M_PI_2*i)/IRISLINES)) + featpts[3][5][1];
     pg[(2*i)+1][2] = featpts[3][5][2] + lfaps[framenum][27];
     rotpoint(pg[(2*i)+1],eyectr,lfaps[framenum][25],lfaps[framenum][23],0.0);
     rotpoint(pg[(2*i)+1],featpts[7][1],lfaps[framenum][48],lfaps[framenum][49],lfaps[framenum][50]);
     glVertex3fv(pg[(2*i)+1]);
   }
   for(i=IRISLINES,j=IRISLINES;i<2*IRISLINES;i++,j--) {
     pg[2*i][0] = -(irisrad*fcos((M_PI_2*j)/IRISLINES)) + featpts[3][5][0];
     pg[2*i][1] = (irisrad*fsin((M_PI_2*j)/IRISLINES)) + featpts[3][5][1];
     pg[2*i][2] = featpts[3][5][2] + lfaps[framenum][27];
     rotpoint(pg[2*i],eyectr,lfaps[framenum][25],lfaps[framenum][23],0.0);
     rotpoint(pg[2*i],featpts[7][1],lfaps[framenum][48],lfaps[framenum][49],lfaps[framenum][50]);
     glVertex3fv(pg[2*i]);
     pg[(2*i)+1][0] = -((lfaps[framenum][29]/2.0)*fcos((M_PI_2*j)/IRISLINES)) + featpts[3][5][0];
     pg[(2*i)+1][1] = ((lfaps[framenum][29]/2.0)*fsin((M_PI_2*j)/IRISLINES)) + featpts[3][5][1];
     pg[(2*i)+1][2] = featpts[3][5][2] + lfaps[framenum][27];
     rotpoint(pg[(2*i)+1],eyectr,lfaps[framenum][25],lfaps[framenum][23],0.0);
     rotpoint(pg[(2*i)+1],featpts[7][1],lfaps[framenum][48],lfaps[framenum][49],lfaps[framenum][50]);
     glVertex3fv(pg[(2*i)+1]);
   }
   for(i=2*IRISLINES,j=0;i<3*IRISLINES;i++,j++) {
     pg[2*i][0] = -(irisrad*fcos((M_PI_2*j)/IRISLINES)) + featpts[3][5][0];
     pg[2*i][1] = -(irisrad*fsin((M_PI_2*j)/IRISLINES)) + featpts[3][5][1];
     pg[2*i][2] = featpts[3][5][2] + lfaps[framenum][27];
     rotpoint(pg[2*i],eyectr,lfaps[framenum][25],lfaps[framenum][23],0.0);
     rotpoint(pg[2*i],featpts[7][1],lfaps[framenum][48],lfaps[framenum][49],lfaps[framenum][50]);
     glVertex3fv(pg[2*i]);
     pg[(2*i)+1][0] = -((lfaps[framenum][29]/2.0)*fcos((M_PI_2*j)/IRISLINES)) + featpts[3][5][0];
     pg[(2*i)+1][1] = -((lfaps[framenum][29]/2.0)*fsin((M_PI_2*j)/IRISLINES)) + featpts[3][5][1];
     pg[(2*i)+1][2] = featpts[3][5][2] + lfaps[framenum][27];
     rotpoint(pg[(2*i)+1],eyectr,lfaps[framenum][25],lfaps[framenum][23],0.0);
     rotpoint(pg[(2*i)+1],featpts[7][1],lfaps[framenum][48],lfaps[framenum][49],lfaps[framenum][50]);
     glVertex3fv(pg[(2*i)+1]);
   }
   for(i=3*IRISLINES,j=IRISLINES;i<4*IRISLINES;i++,j--) {
     pg[2*i][0] = (irisrad*fcos((M_PI_2*j)/IRISLINES)) + featpts[3][5][0];
     pg[2*i][1] = -(irisrad*fsin((M_PI_2*j)/IRISLINES)) + featpts[3][5][1];
     pg[2*i][2] = featpts[3][5][2] + lfaps[framenum][27];
     rotpoint(pg[2*i],eyectr,lfaps[framenum][25],lfaps[framenum][23],0.0);
     rotpoint(pg[2*i],featpts[7][1],lfaps[framenum][48],lfaps[framenum][49],lfaps[framenum][50]);
     glVertex3fv(pg[2*i]);
     pg[(2*i)+1][0] = ((lfaps[framenum][29]/2.0)*fcos((M_PI_2*j)/IRISLINES)) + featpts[3][5][0];
     pg[(2*i)+1][1] = -((lfaps[framenum][29]/2.0)*fsin((M_PI_2*j)/IRISLINES)) + featpts[3][5][1];
     pg[(2*i)+1][2] = featpts[3][5][2] + lfaps[framenum][27];
     rotpoint(pg[(2*i)+1],eyectr,lfaps[framenum][25],lfaps[framenum][23],0.0);
     rotpoint(pg[(2*i)+1],featpts[7][1],lfaps[framenum][48],lfaps[framenum][49],lfaps[framenum][50]);
     glVertex3fv(pg[(2*i)+1]);
   }
   if(isfeatpnt[3][1] && isfeatpnt[3][3])  glEnd();
   /*right iris and pupil*/
   if(isfeatpnt[3][12] && isfeatpnt[3][8]) {/* assume that eyeball diameter is same as eyelid width*/
     eyerad = fabs(featpts[3][12][0] - featpts[3][8][0])/2.0;
   }
   if(isfeatpnt[3][2] && isfeatpnt[3][4]) {
     irisrad = fabs(featpts[3][2][1] - featpts[3][4][1])/2.0;
     glBegin(GL_LINE_LOOP);
   }
   else if(!framenum) printf("WARNING: missing group 3, points 2 and 4\n");
   for(i=0;i<3;i++) eyectr[i] = featpts[3][6][i];
   eyectr[2] -= eyerad;
   for(i=0;i<IRISLINES;i++) {
     pg[2*i][0] = (irisrad*fcos((M_PI_2*i)/IRISLINES)) + featpts[3][6][0];
     pg[2*i][1] = (irisrad*fsin((M_PI_2*i)/IRISLINES)) + featpts[3][6][1];
     pg[2*i][2] = featpts[3][6][2] + lfaps[framenum][28];
     rotpoint(pg[2*i],eyectr,lfaps[framenum][26],lfaps[framenum][24],0.0);
     rotpoint(pg[2*i],featpts[7][1],lfaps[framenum][48],lfaps[framenum][49],lfaps[framenum][50]);
     glVertex3fv(pg[2*i]);
     pg[(2*i)+1][0] = ((lfaps[framenum][29]/2.0)*fcos((M_PI_2*i)/IRISLINES)) + featpts[3][6][0];
     pg[(2*i)+1][1] = ((lfaps[framenum][29]/2.0)*fsin((M_PI_2*i)/IRISLINES)) + featpts[3][6][1];
     pg[(2*i)+1][2] = featpts[3][6][2] + lfaps[framenum][28];
     rotpoint(pg[(2*i)+1],eyectr,lfaps[framenum][26],lfaps[framenum][24],0.0);
     rotpoint(pg[(2*i)+1],featpts[7][1],lfaps[framenum][48],lfaps[framenum][49],lfaps[framenum][50]);
     glVertex3fv(pg[(2*i)+1]);
   }
   for(i=IRISLINES,j=IRISLINES;i<2*IRISLINES;i++,j--) {
     pg[2*i][0] = -(irisrad*fcos((M_PI_2*j)/IRISLINES)) + featpts[3][6][0];
     pg[2*i][1] = (irisrad*fsin((M_PI_2*j)/IRISLINES)) + featpts[3][6][1];
     pg[2*i][2] = featpts[3][6][2] + lfaps[framenum][28];
     rotpoint(pg[2*i],eyectr,lfaps[framenum][26],lfaps[framenum][24],0.0);
     rotpoint(pg[2*i],featpts[7][1],lfaps[framenum][48],lfaps[framenum][49],lfaps[framenum][50]);
     glVertex3fv(pg[2*i]);
     pg[(2*i)+1][0] = -((lfaps[framenum][29]/2.0)*fcos((M_PI_2*j)/IRISLINES)) + featpts[3][6][0];
     pg[(2*i)+1][1] = ((lfaps[framenum][29]/2.0)*fsin((M_PI_2*j)/IRISLINES)) + featpts[3][6][1];
     pg[(2*i)+1][2] = featpts[3][6][2] + lfaps[framenum][28];
     rotpoint(pg[(2*i)+1],eyectr,lfaps[framenum][26],lfaps[framenum][24],0.0);
     rotpoint(pg[(2*i)+1],featpts[7][1],lfaps[framenum][48],lfaps[framenum][49],lfaps[framenum][50]);
     glVertex3fv(pg[(2*i)+1]);
   }
   for(i=2*IRISLINES,j=0;i<3*IRISLINES;i++,j++) {
     pg[2*i][0] = -(irisrad*fcos((M_PI_2*j)/IRISLINES)) + featpts[3][6][0];
     pg[2*i][1] = -(irisrad*fsin((M_PI_2*j)/IRISLINES)) + featpts[3][6][1];
     pg[2*i][2] = featpts[3][6][2] + lfaps[framenum][28];
     rotpoint(pg[2*i],eyectr,lfaps[framenum][26],lfaps[framenum][24],0.0);
     rotpoint(pg[2*i],featpts[7][1],lfaps[framenum][48],lfaps[framenum][49],lfaps[framenum][50]);
     glVertex3fv(pg[2*i]);
     pg[(2*i)+1][0] = -((lfaps[framenum][29]/2.0)*fcos((M_PI_2*j)/IRISLINES)) + featpts[3][6][0];
     pg[(2*i)+1][1] = -((lfaps[framenum][29]/2.0)*fsin((M_PI_2*j)/IRISLINES)) + featpts[3][6][1];
     pg[(2*i)+1][2] = featpts[3][6][2] + lfaps[framenum][28];
     rotpoint(pg[(2*i)+1],eyectr,lfaps[framenum][26],lfaps[framenum][24],0.0);
     rotpoint(pg[(2*i)+1],featpts[7][1],lfaps[framenum][48],lfaps[framenum][49],lfaps[framenum][50]);
     glVertex3fv(pg[(2*i)+1]);
   }
   for(i=3*IRISLINES,j=IRISLINES;i<4*IRISLINES;i++,j--) {
     pg[2*i][0] = (irisrad*fcos((M_PI_2*j)/IRISLINES)) + featpts[3][6][0];
     pg[2*i][1] = -(irisrad*fsin((M_PI_2*j)/IRISLINES)) + featpts[3][6][1];
     pg[2*i][2] = featpts[3][6][2] + lfaps[framenum][28];
     rotpoint(pg[2*i],eyectr,lfaps[framenum][26],lfaps[framenum][24],0.0);
     rotpoint(pg[2*i],featpts[7][1],lfaps[framenum][48],lfaps[framenum][49],lfaps[framenum][50]);
     glVertex3fv(pg[2*i]);
     pg[(2*i)+1][0] = ((lfaps[framenum][29]/2.0)*fcos((M_PI_2*j)/IRISLINES)) + featpts[3][6][0];
     pg[(2*i)+1][1] = -((lfaps[framenum][29]/2.0)*fsin((M_PI_2*j)/IRISLINES)) + featpts[3][6][1];
     pg[(2*i)+1][2] = featpts[3][6][2] + lfaps[framenum][28];
     rotpoint(pg[(2*i)+1],eyectr,lfaps[framenum][26],lfaps[framenum][24],0.0);
     rotpoint(pg[(2*i)+1],featpts[7][1],lfaps[framenum][48],lfaps[framenum][49],lfaps[framenum][50]);
     glVertex3fv(pg[(2*i)+1]);
   }
   if(isfeatpnt[3][2] && isfeatpnt[3][4])  glEnd();
   /*left outer eyelid*/
   glColor3f(.57,.365,.24);
   i = 0;
   grpnum = 3;
   pntnum = 13;
   if(isfeatpnt[grpnum][pntnum]) {
     glBegin(GL_LINE_LOOP);
     pg[i][0] = featpts[grpnum][pntnum][0];
     pg[i][1] = featpts[grpnum][pntnum][1];
     pg[i][2] = featpts[grpnum][pntnum][2];
     rotpoint(pg[i],featpts[7][1],lfaps[framenum][48],lfaps[framenum][49],lfaps[framenum][50]);
     glVertex3fv(pg[i++]);
   }
   else if(!framenum) printf("WARNING: missing group %d, point %d\n",grpnum,pntnum);
   pntnum = 7;
   if(isfeatpnt[grpnum][pntnum]) {
     pg[i][0] = featpts[grpnum][pntnum][0];
     pg[i][1] = featpts[grpnum][pntnum][1];
     pg[i][2] = featpts[grpnum][pntnum][2];
     rotpoint(pg[i],featpts[7][1],lfaps[framenum][48],lfaps[framenum][49],lfaps[framenum][50]);
     glVertex3fv(pg[i++]);
   }
   pntnum = 9;
   if(isfeatpnt[grpnum][pntnum]) {
     pg[i][0] = featpts[grpnum][pntnum][0];
     pg[i][1] = featpts[grpnum][pntnum][1];
     pg[i][2] = featpts[grpnum][pntnum][2];
     rotpoint(pg[i],featpts[7][1],lfaps[framenum][48],lfaps[framenum][49],lfaps[framenum][50]);
     glVertex3fv(pg[i++]);
   }
   pntnum = 11;
   if(isfeatpnt[grpnum][pntnum]) {
     pg[i][0] = featpts[grpnum][pntnum][0];
     pg[i][1] = featpts[grpnum][pntnum][1];
     pg[i][2] = featpts[grpnum][pntnum][2];
     rotpoint(pg[i],featpts[7][1],lfaps[framenum][48],lfaps[framenum][49],lfaps[framenum][50]);
     glVertex3fv(pg[i++]);
   }
   if(isfeatpnt[3][13]) glEnd();
   /*right outer eyelid*/
   i = 0;
   grpnum = 3;
   pntnum = 14;
   if(isfeatpnt[grpnum][pntnum]) {
     glBegin(GL_LINE_LOOP);
     pg[i][0] = featpts[grpnum][pntnum][0];
     pg[i][1] = featpts[grpnum][pntnum][1];
     pg[i][2] = featpts[grpnum][pntnum][2];
     rotpoint(pg[i],featpts[7][1],lfaps[framenum][48],lfaps[framenum][49],lfaps[framenum][50]);
     glVertex3fv(pg[i++]);
   }
   else if(!framenum) printf("WARNING: missing group %d, point %d\n",grpnum,pntnum);
   pntnum = 8;
   if(isfeatpnt[grpnum][pntnum]) {
     pg[i][0] = featpts[grpnum][pntnum][0];
     pg[i][1] = featpts[grpnum][pntnum][1];
     pg[i][2] = featpts[grpnum][pntnum][2];
     rotpoint(pg[i],featpts[7][1],lfaps[framenum][48],lfaps[framenum][49],lfaps[framenum][50]);
     glVertex3fv(pg[i++]);
   }
   pntnum = 10;
   if(isfeatpnt[grpnum][pntnum]) {
     pg[i][0] = featpts[grpnum][pntnum][0];
     pg[i][1] = featpts[grpnum][pntnum][1];
     pg[i][2] = featpts[grpnum][pntnum][2];
     rotpoint(pg[i],featpts[7][1],lfaps[framenum][48],lfaps[framenum][49],lfaps[framenum][50]);
     glVertex3fv(pg[i++]);
   }
   pntnum = 12;
   if(isfeatpnt[grpnum][pntnum]) {
     pg[i][0] = featpts[grpnum][pntnum][0];
     pg[i][1] = featpts[grpnum][pntnum][1];
     pg[i][2] = featpts[grpnum][pntnum][2];
     rotpoint(pg[i],featpts[7][1],lfaps[framenum][48],lfaps[framenum][49],lfaps[framenum][50]);
     glVertex3fv(pg[i++]);
   }
   if(isfeatpnt[3][14]) glEnd();
   /*left inner eyelid*/
   i = 0;
   grpnum = 3;
   pntnum = 1;
   if(isfeatpnt[grpnum][pntnum]) {
     glBegin(GL_LINE_LOOP);
     pg[i][0] = featpts[grpnum][pntnum][0];
     pg[i][1] = featpts[grpnum][pntnum][1] - lfaps[framenum][19];
     pg[i][2] = featpts[grpnum][pntnum][2];
     rotpoint(pg[i],featpts[7][1],lfaps[framenum][48],lfaps[framenum][49],lfaps[framenum][50]);
     glVertex3fv(pg[i++]);
   }
   else if(!framenum) printf("WARNING: missing group %d, point %d\n",grpnum,pntnum);
   pntnum = 7;
   if(isfeatpnt[grpnum][pntnum]) {
     pg[i][0] = featpts[grpnum][pntnum][0];
     pg[i][1] = featpts[grpnum][pntnum][1];
     pg[i][2] = featpts[grpnum][pntnum][2];
     rotpoint(pg[i],featpts[7][1],lfaps[framenum][48],lfaps[framenum][49],lfaps[framenum][50]);
     glVertex3fv(pg[i++]);
   }
   pntnum = 3;
   if(isfeatpnt[grpnum][pntnum]) {
     pg[i][0] = featpts[grpnum][pntnum][0];
     pg[i][1] = featpts[grpnum][pntnum][1] + lfaps[framenum][21];
     pg[i][2] = featpts[grpnum][pntnum][2];
     rotpoint(pg[i],featpts[7][1],lfaps[framenum][48],lfaps[framenum][49],lfaps[framenum][50]);
     glVertex3fv(pg[i++]);
   }
   pntnum = 11;
   if(isfeatpnt[grpnum][pntnum]) {
     pg[i][0] = featpts[grpnum][pntnum][0];
     pg[i][1] = featpts[grpnum][pntnum][1];
     pg[i][2] = featpts[grpnum][pntnum][2];
     rotpoint(pg[i],featpts[7][1],lfaps[framenum][48],lfaps[framenum][49],lfaps[framenum][50]);
     glVertex3fv(pg[i++]);
   }
   if(isfeatpnt[3][1]) glEnd();
   /*right inner eyelid*/
   i = 0;
   grpnum = 3;
   pntnum = 2;
   if(isfeatpnt[grpnum][pntnum]) {
     glBegin(GL_LINE_LOOP);
     pg[i][0] = featpts[grpnum][pntnum][0];
     pg[i][1] = featpts[grpnum][pntnum][1] - lfaps[framenum][20];
     pg[i][2] = featpts[grpnum][pntnum][2];
     rotpoint(pg[i],featpts[7][1],lfaps[framenum][48],lfaps[framenum][49],lfaps[framenum][50]);
     glVertex3fv(pg[i++]);
   }
   else if(!framenum) printf("WARNING: missing group %d, point %d\n",grpnum,pntnum);
   pntnum = 8;
   if(isfeatpnt[grpnum][pntnum]) {
     pg[i][0] = featpts[grpnum][pntnum][0];
     pg[i][1] = featpts[grpnum][pntnum][1];
     pg[i][2] = featpts[grpnum][pntnum][2];
     rotpoint(pg[i],featpts[7][1],lfaps[framenum][48],lfaps[framenum][49],lfaps[framenum][50]);
     glVertex3fv(pg[i++]);
   }
   pntnum = 4;
   if(isfeatpnt[grpnum][pntnum]) {
     pg[i][0] = featpts[grpnum][pntnum][0];
     pg[i][1] = featpts[grpnum][pntnum][1] + lfaps[framenum][22];
     pg[i][2] = featpts[grpnum][pntnum][2];
     rotpoint(pg[i],featpts[7][1],lfaps[framenum][48],lfaps[framenum][49],lfaps[framenum][50]);
     glVertex3fv(pg[i++]);
   }
   pntnum = 12;
   if(isfeatpnt[grpnum][pntnum]) {
     pg[i][0] = featpts[grpnum][pntnum][0];
     pg[i][1] = featpts[grpnum][pntnum][1];
     pg[i][2] = featpts[grpnum][pntnum][2];
     rotpoint(pg[i],featpts[7][1],lfaps[framenum][48],lfaps[framenum][49],lfaps[framenum][50]);
     glVertex3fv(pg[i++]);
   }
   if(isfeatpnt[3][2]) glEnd();
   /*left eyebrow*/
   glColor3f(.35,.31,.078);
   i = 0;
   grpnum = 4;
   pntnum = 1;
   if(isfeatpnt[grpnum][pntnum]) {
     glBegin(GL_LINE_STRIP);
     pg[i][0] = featpts[grpnum][pntnum][0] - lfaps[framenum][37];
     pg[i][1] = featpts[grpnum][pntnum][1] + lfaps[framenum][31];
     pg[i][2] = featpts[grpnum][pntnum][2];
     rotpoint(pg[i],featpts[7][1],lfaps[framenum][48],lfaps[framenum][49],lfaps[framenum][50]);
     glVertex3fv(pg[i++]);
   }
   else if(!framenum) printf("WARNING: missing group %d, point %d\n",grpnum,pntnum);
   pntnum = 3;
   if(isfeatpnt[grpnum][pntnum]) {
     pg[i][0] = featpts[grpnum][pntnum][0] - (lfaps[framenum][37]/2.0);/*interpolate*/
     pg[i][1] = featpts[grpnum][pntnum][1] + lfaps[framenum][33];
     pg[i][2] = featpts[grpnum][pntnum][2];
     rotpoint(pg[i],featpts[7][1],lfaps[framenum][48],lfaps[framenum][49],lfaps[framenum][50]);
     glVertex3fv(pg[i++]);
   }
   pntnum = 5;
   if(isfeatpnt[grpnum][pntnum]) {
     pg[i][0] = featpts[grpnum][pntnum][0];
     pg[i][1] = featpts[grpnum][pntnum][1] + lfaps[framenum][35];
     pg[i][2] = featpts[grpnum][pntnum][2];
     rotpoint(pg[i],featpts[7][1],lfaps[framenum][48],lfaps[framenum][49],lfaps[framenum][50]);
     glVertex3fv(pg[i++]);
   }
   if(isfeatpnt[4][1]) glEnd();
   /*right eyebrow*/
   i = 0;
   grpnum = 4;
   pntnum = 2;
   if(isfeatpnt[grpnum][pntnum]) {
     glBegin(GL_LINE_STRIP);
     pg[i][0] = featpts[grpnum][pntnum][0] + lfaps[framenum][38];
     pg[i][1] = featpts[grpnum][pntnum][1] + lfaps[framenum][32];
     pg[i][2] = featpts[grpnum][pntnum][2];
     rotpoint(pg[i],featpts[7][1],lfaps[framenum][48],lfaps[framenum][49],lfaps[framenum][50]);
     glVertex3fv(pg[i++]);
   }
   else if(!framenum) printf("WARNING: missing group %d, point %d\n",grpnum,pntnum);
   pntnum = 4;
   if(isfeatpnt[grpnum][pntnum]) {
     pg[i][0] = featpts[grpnum][pntnum][0] + (lfaps[framenum][38]/2.0);
     pg[i][1] = featpts[grpnum][pntnum][1] + lfaps[framenum][34];
     pg[i][2] = featpts[grpnum][pntnum][2];
     rotpoint(pg[i],featpts[7][1],lfaps[framenum][48],lfaps[framenum][49],lfaps[framenum][50]);
     glVertex3fv(pg[i++]);
   }
   pntnum = 6;
   if(isfeatpnt[grpnum][pntnum]) {
     pg[i][0] = featpts[grpnum][pntnum][0];
     pg[i][1] = featpts[grpnum][pntnum][1] + lfaps[framenum][36];
     pg[i][2] = featpts[grpnum][pntnum][2];
     rotpoint(pg[i],featpts[7][1],lfaps[framenum][48],lfaps[framenum][49],lfaps[framenum][50]);
     glVertex3fv(pg[i++]);
   }
   if(isfeatpnt[4][2]) glEnd();
   /*nose*/
   glColor3f(.57,.365,.24);
   i = 0;
   grpnum = 9;
   pntnum = 7;
   if(isfeatpnt[grpnum][pntnum]) {
     glBegin(GL_LINE_STRIP);
     pg[i][0] = featpts[grpnum][pntnum][0];
     pg[i][1] = featpts[grpnum][pntnum][1];
     pg[i][2] = featpts[grpnum][pntnum][2];
     rotpoint(pg[i],featpts[7][1],lfaps[framenum][48],lfaps[framenum][49],lfaps[framenum][50]);
     glVertex3fv(pg[i++]);
   }
   else if(!framenum) printf("WARNING: missing group %d, point %d\n",grpnum,pntnum);
   pntnum = 13;
   if(isfeatpnt[grpnum][pntnum]) {
     pg[i][0] = featpts[grpnum][pntnum][0];
     pg[i][1] = featpts[grpnum][pntnum][1];
     pg[i][2] = featpts[grpnum][pntnum][2];
     rotpoint(pg[i],featpts[7][1],lfaps[framenum][48],lfaps[framenum][49],lfaps[framenum][50]);
     glVertex3fv(pg[i++]);
   }
   pntnum = 5;
   if(isfeatpnt[grpnum][pntnum]) {
     pg[i][0] = featpts[grpnum][pntnum][0];
     pg[i][1] = featpts[grpnum][pntnum][1];
     pg[i][2] = featpts[grpnum][pntnum][2];
     rotpoint(pg[i],featpts[7][1],lfaps[framenum][48],lfaps[framenum][49],lfaps[framenum][50]);
     glVertex3fv(pg[i++]);
   }
   pntnum = 1;
   if(isfeatpnt[grpnum][pntnum]) {
     pg[i][0] = featpts[grpnum][pntnum][0] + lfaps[framenum][61];
     pg[i][1] = featpts[grpnum][pntnum][1];
     pg[i][2] = featpts[grpnum][pntnum][2];
     rotpoint(pg[i],featpts[7][1],lfaps[framenum][48],lfaps[framenum][49],lfaps[framenum][50]);
     glVertex3fv(pg[i++]);
   }
   pntnum = 3;
   if(isfeatpnt[grpnum][pntnum]) {
     pg[i][0] = featpts[grpnum][pntnum][0] - lfaps[framenum][64];
     pg[i][1] = featpts[grpnum][pntnum][1] + lfaps[framenum][63];
     pg[i][2] = featpts[grpnum][pntnum][2];
     rotpoint(pg[i],featpts[7][1],lfaps[framenum][48],lfaps[framenum][49],lfaps[framenum][50]);
     glVertex3fv(pg[i++]);
   }
   pntnum = 2;
   if(isfeatpnt[grpnum][pntnum]) {
     pg[i][0] = featpts[grpnum][pntnum][0] - lfaps[framenum][62];
     pg[i][1] = featpts[grpnum][pntnum][1];
     pg[i][2] = featpts[grpnum][pntnum][2];
     rotpoint(pg[i],featpts[7][1],lfaps[framenum][48],lfaps[framenum][49],lfaps[framenum][50]);
     glVertex3fv(pg[i++]);
   }
   pntnum = 4;
   if(isfeatpnt[grpnum][pntnum]) {
     pg[i][0] = featpts[grpnum][pntnum][0];
     pg[i][1] = featpts[grpnum][pntnum][1];
     pg[i][2] = featpts[grpnum][pntnum][2];
     rotpoint(pg[i],featpts[7][1],lfaps[framenum][48],lfaps[framenum][49],lfaps[framenum][50]);
     glVertex3fv(pg[i++]);
   }

   pntnum = 14;
   if(isfeatpnt[grpnum][pntnum]) {
     pg[i][0] = featpts[grpnum][pntnum][0];
     pg[i][1] = featpts[grpnum][pntnum][1];
     pg[i][2] = featpts[grpnum][pntnum][2];
     rotpoint(pg[i],featpts[7][1],lfaps[framenum][48],lfaps[framenum][49],lfaps[framenum][50]);
     glVertex3fv(pg[i++]);
   }
   pntnum = 6;
   if(isfeatpnt[grpnum][pntnum]) {
     pg[i][0] = featpts[grpnum][pntnum][0];
     pg[i][1] = featpts[grpnum][pntnum][1];
     pg[i][2] = featpts[grpnum][pntnum][2];
     rotpoint(pg[i],featpts[7][1],lfaps[framenum][48],lfaps[framenum][49],lfaps[framenum][50]);
     glVertex3fv(pg[i++]);
   }
   if(isfeatpnt[9][7]) glEnd();
   /*nose bridge and centerline*/
   i = 0;
   pntnum = 15;
   if(isfeatpnt[grpnum][pntnum]) {
     glBegin(GL_LINE_STRIP);
     pg[i][0] = featpts[grpnum][pntnum][0];
     pg[i][1] = featpts[grpnum][pntnum][1];
     pg[i][2] = featpts[grpnum][pntnum][2];
     rotpoint(pg[i],featpts[7][1],lfaps[framenum][48],lfaps[framenum][49],lfaps[framenum][50]);
     glVertex3fv(pg[i++]);
   }
   else if(!framenum) printf("WARNING: missing group %d, point %d\n",grpnum,pntnum);
   pntnum = 3;
   if(isfeatpnt[grpnum][pntnum]) {
     pg[i][0] = featpts[grpnum][pntnum][0];
     pg[i][1] = featpts[grpnum][pntnum][1];
     pg[i][2] = featpts[grpnum][pntnum][2];
     rotpoint(pg[i],featpts[7][1],lfaps[framenum][48],lfaps[framenum][49],lfaps[framenum][50]);
     glVertex3fv(pg[i++]);
   }
   pntnum = 12;
   if(isfeatpnt[grpnum][pntnum]) {
     pg[i][0] = featpts[grpnum][pntnum][0];
     pg[i][1] = featpts[grpnum][pntnum][1];
     pg[i][2] = featpts[grpnum][pntnum][2];
     rotpoint(pg[i],featpts[7][1],lfaps[framenum][48],lfaps[framenum][49],lfaps[framenum][50]);
     glVertex3fv(pg[i++]);
   }
   if(isfeatpnt[9][15]) glEnd();
   /*left ear*/
   i = 0;
   grpnum = 10;
   pntnum = 9;
   if(isfeatpnt[grpnum][pntnum]) {
     glBegin(GL_LINE_STRIP);
     pg[i][0] = featpts[grpnum][pntnum][0];
     pg[i][1] = featpts[grpnum][pntnum][1];
     pg[i][2] = featpts[grpnum][pntnum][2];
     rotpoint(pg[i],featpts[7][1],lfaps[framenum][48],lfaps[framenum][49],lfaps[framenum][50]);
     glVertex3fv(pg[i++]);
   }
   else if(!framenum) printf("WARNING: missing group %d, point %d\n",grpnum,pntnum);
   pntnum = 1;
   if(isfeatpnt[grpnum][pntnum]) {
     pg[i][0] = featpts[grpnum][pntnum][0];
     pg[i][1] = featpts[grpnum][pntnum][1] + lfaps[framenum][65];
     pg[i][2] = featpts[grpnum][pntnum][2];
     rotpoint(pg[i],featpts[7][1],lfaps[framenum][48],lfaps[framenum][49],lfaps[framenum][50]);
     glVertex3fv(pg[i++]);
   }
   pntnum = 3;
   if(isfeatpnt[grpnum][pntnum]) {
     pg[i][0] = featpts[grpnum][pntnum][0]  + lfaps[framenum][67];
     pg[i][1] = featpts[grpnum][pntnum][1];
     pg[i][2] = featpts[grpnum][pntnum][2];
     rotpoint(pg[i],featpts[7][1],lfaps[framenum][48],lfaps[framenum][49],lfaps[framenum][50]);
     glVertex3fv(pg[i++]);
   }
   pntnum = 5;
   if(isfeatpnt[grpnum][pntnum]) {
     pg[i][0] = featpts[grpnum][pntnum][0];
     pg[i][1] = featpts[grpnum][pntnum][1];
     pg[i][2] = featpts[grpnum][pntnum][2];
     rotpoint(pg[i],featpts[7][1],lfaps[framenum][48],lfaps[framenum][49],lfaps[framenum][50]);
     glVertex3fv(pg[i++]);
   }
   pntnum = 7;
   if(isfeatpnt[grpnum][pntnum]) {
     pg[i][0] = featpts[grpnum][pntnum][0];
     pg[i][1] = featpts[grpnum][pntnum][1];
     pg[i][2] = featpts[grpnum][pntnum][2];
     rotpoint(pg[i],featpts[7][1],lfaps[framenum][48],lfaps[framenum][49],lfaps[framenum][50]);
     glVertex3fv(pg[i++]);
   }
   if(isfeatpnt[10][9]) glEnd();
   /*right ear*/
   i = 0;
   grpnum = 10;
   pntnum = 10;
   if(isfeatpnt[grpnum][pntnum]) {
     glBegin(GL_LINE_STRIP);
     pg[i][0] = featpts[grpnum][pntnum][0];
     pg[i][1] = featpts[grpnum][pntnum][1];
     pg[i][2] = featpts[grpnum][pntnum][2];
     rotpoint(pg[i],featpts[7][1],lfaps[framenum][48],lfaps[framenum][49],lfaps[framenum][50]);
     glVertex3fv(pg[i++]);
   }
   else if(!framenum) printf("WARNING: missing group %d, point %d\n",grpnum,pntnum);
   pntnum = 2;
   if(isfeatpnt[grpnum][pntnum]) {
     pg[i][0] = featpts[grpnum][pntnum][0];
     pg[i][1] = featpts[grpnum][pntnum][1] + lfaps[framenum][66];
     pg[i][2] = featpts[grpnum][pntnum][2];
     rotpoint(pg[i],featpts[7][1],lfaps[framenum][48],lfaps[framenum][49],lfaps[framenum][50]);
     glVertex3fv(pg[i++]);
   }
   pntnum = 4;
   if(isfeatpnt[grpnum][pntnum]) {
     pg[i][0] = featpts[grpnum][pntnum][0] - lfaps[framenum][68];
     pg[i][1] = featpts[grpnum][pntnum][1];
     pg[i][2] = featpts[grpnum][pntnum][2];
     rotpoint(pg[i],featpts[7][1],lfaps[framenum][48],lfaps[framenum][49],lfaps[framenum][50]);
     glVertex3fv(pg[i++]);
   }
   pntnum = 6;
   if(isfeatpnt[grpnum][pntnum]) {
     pg[i][0] = featpts[grpnum][pntnum][0];
     pg[i][1] = featpts[grpnum][pntnum][1];
     pg[i][2] = featpts[grpnum][pntnum][2];
     rotpoint(pg[i],featpts[7][1],lfaps[framenum][48],lfaps[framenum][49],lfaps[framenum][50]);
     glVertex3fv(pg[i++]);
   }
   pntnum = 8;
   if(isfeatpnt[grpnum][pntnum]) {
     pg[i][0] = featpts[grpnum][pntnum][0];
     pg[i][1] = featpts[grpnum][pntnum][1];
     pg[i][2] = featpts[grpnum][pntnum][2];
     rotpoint(pg[i],featpts[7][1],lfaps[framenum][48],lfaps[framenum][49],lfaps[framenum][50]);
     glVertex3fv(pg[i++]);
   }
   if(isfeatpnt[10][10]) glEnd();
   /*top of head*/
   glColor3f(.35,.31,.078);
   i = 0;
   grpnum = 10;
   pntnum = 10;
   if(isfeatpnt[grpnum][pntnum]) {
     glBegin(GL_LINE_STRIP);
     pg[i][0] = featpts[grpnum][pntnum][0];
     pg[i][1] = featpts[grpnum][pntnum][1];
     pg[i][2] = featpts[grpnum][pntnum][2];
     rotpoint(pg[i],featpts[7][1],lfaps[framenum][48],lfaps[framenum][49],lfaps[framenum][50]);
     glVertex3fv(pg[i++]);
   }
   else if(!framenum) printf("WARNING: missing group %d, point %d\n",grpnum,pntnum);
   grpnum = 11;
   pntnum = 2;
   if(isfeatpnt[grpnum][pntnum]) {
     pg[i][0] = featpts[grpnum][pntnum][0];
     pg[i][1] = featpts[grpnum][pntnum][1];
     pg[i][2] = featpts[grpnum][pntnum][2];
     rotpoint(pg[i],featpts[7][1],lfaps[framenum][48],lfaps[framenum][49],lfaps[framenum][50]);
     glVertex3fv(pg[i++]);
   }
   pntnum = 1;
   if(isfeatpnt[grpnum][pntnum]) {
     pg[i][0] = featpts[grpnum][pntnum][0];
     pg[i][1] = featpts[grpnum][pntnum][1];
     pg[i][2] = featpts[grpnum][pntnum][2];
     rotpoint(pg[i],featpts[7][1],lfaps[framenum][48],lfaps[framenum][49],lfaps[framenum][50]);
     glVertex3fv(pg[i++]);
   }
   pntnum = 3;
   if(isfeatpnt[grpnum][pntnum]) {
     pg[i][0] = featpts[grpnum][pntnum][0];
     pg[i][1] = featpts[grpnum][pntnum][1];
     pg[i][2] = featpts[grpnum][pntnum][2];
     rotpoint(pg[i],featpts[7][1],lfaps[framenum][48],lfaps[framenum][49],lfaps[framenum][50]);
     glVertex3fv(pg[i++]);
   }
   grpnum = 10;
   pntnum = 9;
   if(isfeatpnt[grpnum][pntnum]) {
     pg[i][0] = featpts[grpnum][pntnum][0];
     pg[i][1] = featpts[grpnum][pntnum][1];
     pg[i][2] = featpts[grpnum][pntnum][2];
     rotpoint(pg[i],featpts[7][1],lfaps[framenum][48],lfaps[framenum][49],lfaps[framenum][50]);
     glVertex3fv(pg[i++]);
   }
   if(isfeatpnt[10][10]) glEnd();
   /*cheek*/
   glColor3f(.57,.365,.24);
   i = 0;
   grpnum = 5;
   pntnum = 1;
   if(isfeatpnt[grpnum][pntnum]) {
     glBegin(GL_POINTS);
     pg[i][0] = featpts[grpnum][pntnum][0] + lfaps[framenum][39];
     pg[i][1] = featpts[grpnum][pntnum][1] + lfaps[framenum][41];
     pg[i][2] = featpts[grpnum][pntnum][2];
     rotpoint(pg[i],featpts[7][1],lfaps[framenum][48],lfaps[framenum][49],lfaps[framenum][50]);
     glVertex3fv(pg[i++]);
   }
   else if(!framenum) printf("WARNING: missing group %d, point %d\n",grpnum,pntnum);
   pntnum = 2;
   if(isfeatpnt[grpnum][pntnum]) {
     pg[i][0] = featpts[grpnum][pntnum][0] + lfaps[framenum][40];
     pg[i][1] = featpts[grpnum][pntnum][1] + lfaps[framenum][42];
     pg[i][2] = featpts[grpnum][pntnum][2];
     rotpoint(pg[i],featpts[7][1],lfaps[framenum][48],lfaps[framenum][49],lfaps[framenum][50]);
     glVertex3fv(pg[i++]);
   }
   pntnum = 3;
   if(isfeatpnt[grpnum][pntnum]) {
     pg[i][0] = featpts[grpnum][pntnum][0];
     pg[i][1] = featpts[grpnum][pntnum][1];
     pg[i][2] = featpts[grpnum][pntnum][2];
     rotpoint(pg[i],featpts[7][1],lfaps[framenum][48],lfaps[framenum][49],lfaps[framenum][50]);
     glVertex3fv(pg[i++]);
   }
   pntnum = 4;
   if(isfeatpnt[grpnum][pntnum]) {
     pg[i][0] = featpts[grpnum][pntnum][0];
     pg[i][1] = featpts[grpnum][pntnum][1];
     pg[i][2] = featpts[grpnum][pntnum][2];
     rotpoint(pg[i],featpts[7][1],lfaps[framenum][48],lfaps[framenum][49],lfaps[framenum][50]);
     glVertex3fv(pg[i++]);
   }
   if(isfeatpnt[5][1]) glEnd();
   /*top of head*/
   glColor3f(.57,.365,.24);
   i = 0;
   grpnum = 11;
   pntnum = 2;
   if(isfeatpnt[grpnum][pntnum] && isfeatpnt[11][4]) {
     glBegin(GL_LINE_STRIP);
     pg[i][0] = featpts[grpnum][pntnum][0];
     pg[i][1] = featpts[grpnum][pntnum][1];
     pg[i][2] = featpts[grpnum][pntnum][2];
     rotpoint(pg[i],featpts[7][1],lfaps[framenum][48],lfaps[framenum][49],lfaps[framenum][50]);
     glVertex3fv(pg[i++]);
   }
   else if(!framenum) printf("WARNING: missing group %d, point %d\n",grpnum,pntnum);
   pntnum = 4;
   if(isfeatpnt[grpnum][pntnum]) {
     pg[i][0] = featpts[grpnum][pntnum][0];
     pg[i][1] = featpts[grpnum][pntnum][1];
     pg[i][2] = featpts[grpnum][pntnum][2];
     rotpoint(pg[i],featpts[7][1],lfaps[framenum][48],lfaps[framenum][49],lfaps[framenum][50]);
     glVertex3fv(pg[i++]);
   }
   pntnum = 3;
   if(isfeatpnt[grpnum][pntnum]) {
     pg[i][0] = featpts[grpnum][pntnum][0];
     pg[i][1] = featpts[grpnum][pntnum][1];
     pg[i][2] = featpts[grpnum][pntnum][2];
     rotpoint(pg[i],featpts[7][1],lfaps[framenum][48],lfaps[framenum][49],lfaps[framenum][50]);
     glVertex3fv(pg[i++]);
   }
   if(isfeatpnt[11][2] && isfeatpnt[11][4]) glEnd();
   /*top of hair*/
   glColor3f(.35,.31,.078);
   i = 0;
   grpnum = 11;
   pntnum = 2;
   if(isfeatpnt[grpnum][pntnum] && isfeatpnt[11][5]) {
     glBegin(GL_LINE_STRIP);
     pg[i][0] = featpts[grpnum][pntnum][0];
     pg[i][1] = featpts[grpnum][pntnum][1];
     pg[i][2] = featpts[grpnum][pntnum][2];
     rotpoint(pg[i],featpts[7][1],lfaps[framenum][48],lfaps[framenum][49],lfaps[framenum][50]);
     glVertex3fv(pg[i++]);
   }
   else if(!framenum) printf("WARNING: missing group %d, point %d\n",grpnum,pntnum);
   pntnum = 5;
   if(isfeatpnt[grpnum][pntnum]) {
     pg[i][0] = featpts[grpnum][pntnum][0];
     pg[i][1] = featpts[grpnum][pntnum][1];
     pg[i][2] = featpts[grpnum][pntnum][2];
     rotpoint(pg[i],featpts[7][1],lfaps[framenum][48],lfaps[framenum][49],lfaps[framenum][50]);
     glVertex3fv(pg[i++]);
   }
   pntnum = 3;
   if(isfeatpnt[grpnum][pntnum]) {
     pg[i][0] = featpts[grpnum][pntnum][0];
     pg[i][1] = featpts[grpnum][pntnum][1];
     pg[i][2] = featpts[grpnum][pntnum][2];
     rotpoint(pg[i],featpts[7][1],lfaps[framenum][48],lfaps[framenum][49],lfaps[framenum][50]);
     glVertex3fv(pg[i++]);
   }
   if(isfeatpnt[11][2] && isfeatpnt[11][5]) glEnd();
}







