
/*
COPYRIGHT STATEMENT

NOTE: This copyright statement applies to all individual files or
modules in which the developers listed below are mentioned.

This software module was originally developed by the Lucent Technologies
Murray Hill, New Jersey, USA. Contributors to the software
include but are not limited to: Eric Petajan and Jialin Zhong.
This software module is an implementation of a part of one or more
MPEG-4 Visual (ISO/IEC 14496-2) tools as specified by MPEG-4 Visual
(ISO/IEC 14496-2). ISO/IEC gives users of the MPEG-4 Visual standard
free license to this software module or modifications thereof for
use in hardware or software products claiming conformance to the
MPEG-4 Visual (ISO/IEC 14496-2) standard. Those intending to use
this software module in hardware or software products are advised
that its use may infringe existing patents. The original developers
of this software and his/her company, the subsequent editors and
their companies, and ISO/IEC have no liability for use of this
software module or modifications thereof. Copyright is not released
for non MPEG-4 Visual standard conforming products. The original
developers retain full right to use the code for its own purpose,
assign or donate the code for non MPEG-4 Visual conforming products.
This copyright notice must be included in all copies or derivative
works.

Copyright (c) 1997

CONTRIBUTION TO THE WG11 PROGRAM MPEG 4 PLAYER
______________________________________________________________________

STANDARDS PROJECT:  MPEG 4 Player
______________________________________________________________________

TITLE:  3D Line-Based Face Animation Module
______________________________________________________________________

AUTHORS:  Eric Petajan, Jialin Zhong

SOURCE:      Lucent Technologies Inc.

CONTACT:        Eric Petajan
                Lucent Technologies
                Room 2B231
                600 Mt. Ave.
                Murray Hill, NJ  07974
                Voice:  +1 908-582-3160
                Fax: +1 908-582-6632
                Email:  edp@bell-labs.com
______________________________________________________________________

ABSTRACT:  this module inputs face animation and definition parameters,
and outputs 3D points, lines, and polygons.
*/

#include <stdio.h>
#include <math.h>
#include "wireface.h"

readfdp(char *fdpfilename, FDPtype *FDPs) {
  FILE *fdpfile;
  char cbuf[1000];
  int grpnum,pntnum,i;
  float dispscale,tmpfp[3];
  float bboxmin[3],bboxmax[3],bboxdim[3];
  int **isfeatpnt;
  float ***featpts;
  
  isfeatpnt = FDPs->isfeatpnt;
  featpts = FDPs->featpts;
  FDPs->dispscale = 4.0;
  FDPs->MWFAPUperpel= 0.1;
  FDPs->MNSFAPUperpel= 0.05;
  FDPs->ENSFAPUperpel= 0.05;
  FDPs->ESFAPUperpel= 0.2;
  FDPs->IRISDFAPUperpel=.001;
  if(!(fdpfile = fopen(fdpfilename,"r"))) printf("ERROR: can't open %s\n",fdpfilename);
  for(i = 0;i<3;i++) {bboxmin[i] = 100000.0;bboxmax[i] = -100000.0;}
  for(grpnum=0;grpnum<16;grpnum++) for(pntnum=0;pntnum<20;pntnum++) isfeatpnt[grpnum][pntnum] = 0;
  while(1) {/*read header line*/
    if(fscanf(fdpfile,"%s",cbuf));
    else {
      printf("ERROR: bad fdpfile: %s\n",cbuf);
      exit(2);
    }
    if(cbuf[0] == '#') while(getc(fdpfile) != '\n');
    else {
      sscanf(cbuf,"%f ",&(FDPs->version));
      if(FDPs->version != 1.0) printf("WARNING: Expecting FDP file version 1.0 not %f\n",FDPs->version);
      while(getc(fdpfile) != '\n');
      break;
    }
  }
  while(1) {
    if(fscanf(fdpfile,"%s",cbuf)!= EOF);
    else {
      break;
    }
    if(cbuf[0] == '#') while(getc(fdpfile) != '\n');
    else {
      sscanf(cbuf,"%d.%d",&grpnum,&pntnum);
      isfeatpnt[grpnum][pntnum] = 1;
      if(fscanf(fdpfile,"%f %f %f\n",&featpts[grpnum][pntnum][0],&featpts[grpnum][pntnum][1],&featpts[grpnum][pntnum][2])!= EOF)
	for(i = 0;i<3;i++) {
	  if(featpts[grpnum][pntnum][i] < bboxmin[i]) bboxmin[i] = featpts[grpnum][pntnum][i];
	  if(featpts[grpnum][pntnum][i] > bboxmax[i]) bboxmax[i] = featpts[grpnum][pntnum][i];
	}
      else break;
    }
  }
  if(isfeatpnt[2][7] && isfeatpnt[2][9]) {
    if(featpts[2][7][1] != featpts[2][9][1]) {
      printf("WARNING: featpts[2][7][1] != featpts[2][9][1]. Using featpts[2][7][1] for both\n");
      featpts[2][7][1] = featpts[2][9][1];
    }
    if(featpts[2][7][0] != featpts[2][9][0]) {
      printf("WARNING: featpts[2][7][0] != featpts[2][9][0]. Using featpts[2][7][0] for both\n");
      featpts[2][7][0] = featpts[2][9][0];
    }
  }
  if(isfeatpnt[2][6] && isfeatpnt[2][8]) {
    if(featpts[2][6][1] != featpts[2][8][1]) {
      printf("WARNING: featpts[2][6][1] != featpts[2][8][1]. Using featpts[2][6][1] for both\n");
      featpts[2][6][1] = featpts[2][8][1];
    }
    if(featpts[2][6][0] != featpts[2][8][0]) {
      printf("WARNING: featpts[2][6][0] != featpts[2][8][0]. Using featpts[2][6][0] for both\n");
      featpts[2][6][0] = featpts[2][8][0];
    }
  }
  if(isfeatpnt[2][2] && isfeatpnt[2][3]) {
    if(featpts[2][2][1] != featpts[2][3][1]) {
      printf("WARNING: featpts[2][2][1] != featpts[2][3][1]. Using featpts[2][2][1] for both\n");
      featpts[2][2][1] = featpts[2][3][1];
    }
    if(featpts[2][2][0] != featpts[2][3][0]) {
      printf("WARNING: featpts[2][2][0] != featpts[2][3][0]. Using featpts[2][2][0] for both\n");
      featpts[2][2][0] = featpts[2][3][0];
    }
  }
  /*interpolate top of spine if not specified*/
  if(!isfeatpnt[7][1]) {
    isfeatpnt[7][1] = 1;
    featpts[7][1][0] = (bboxmin[0] + bboxmax[0])/2.0;
    featpts[7][1][1] = bboxmin[1];
    featpts[7][1][2] = bboxmin[2];
  }
  /*move top of spine to origin*/
  for(i = 0;i<3;i++) {
    tmpfp[i] = featpts[7][1][i];
     bboxmin[i] -= tmpfp[i];
     bboxmax[i] -= tmpfp[i];
  }
  for(grpnum=0;grpnum<16;grpnum++) for(pntnum=0;pntnum<20;pntnum++) if(isfeatpnt[grpnum][pntnum]) for(i = 0;i<3;i++) {
    featpts[grpnum][pntnum][i] -= tmpfp[i];
  }
  /*compute size of bbox*/
  for(i = 0;i<3;i++) bboxdim[i] = bboxmax[i] - bboxmin[i];
  if(bboxdim[0] > bboxdim[1]) {
    if(bboxdim[0] > bboxdim[2]) dispscale = 50.0/bboxdim[0];
    else dispscale = 50.0/bboxdim[2];
  }
  else {
    if(bboxdim[1] > bboxdim[2]) dispscale = 50.0/bboxdim[1];
    else dispscale = 50.0/bboxdim[2];
  }
  for(grpnum=0;grpnum<16;grpnum++) for(pntnum=0;pntnum<20;pntnum++) if(isfeatpnt[grpnum][pntnum]) for(i = 0;i<3;i++) {
        featpts[grpnum][pntnum][i] *= dispscale;/*scale head to fit window*/
  }
  if(isfeatpnt[2][5] && isfeatpnt[2][4]) FDPs->MWFAPUperpel = 1024.0/(fabs((featpts[2][5][0] - featpts[2][4][0])));
  else printf("WARNING: using default MWFAPUperpel = %f\n",FDPs->MWFAPUperpel);
  if(isfeatpnt[2][5] && isfeatpnt[2][4] && isfeatpnt[9][3]) FDPs->MNSFAPUperpel = 1024.0/(fabs(((featpts[2][5][1] + featpts[2][4][1])/2.0) - featpts[9][3][1] ));
  else printf("WARNING: using default MNSFAPUperpel = %f\n",FDPs->MNSFAPUperpel);
  if(isfeatpnt[3][5] && isfeatpnt[3][6]) FDPs->ESFAPUperpel = 1024.0/(fabs(featpts[3][5][0] - featpts[3][6][0]));
  else printf("WARNING: using default ESFAPUperpel = %f\n",FDPs->ESFAPUperpel);
  if(isfeatpnt[3][5] && isfeatpnt[3][6] && isfeatpnt[9][3]) FDPs->ENSFAPUperpel = 1024.0/(fabs(((featpts[3][5][1] + featpts[3][6][1])/2.0) - featpts[9][3][1]));
  else printf("WARNING: using default ENSFAPUperpel = %f\n",FDPs->ENSFAPUperpel);
  if(isfeatpnt[3][2] && isfeatpnt[3][4]) FDPs->IRISDFAPUperpel = 1024.0/(fabs(featpts[3][2][1] - featpts[3][4][1]));
  else printf("WARNING: using default IRISDFAPUperpel = %f\n",FDPs->IRISDFAPUperpel);
}

leftrightinterpolation(FAPtype *FAPs, int frameind, int startfapnum, int endfapnum) {
  int fapnum,i;
  int **fapmask;
  float **lfaps;

  fapmask = FAPs->fapmask;
  lfaps = FAPs->lfaps;

  for(fapnum=startfapnum;fapnum <= endfapnum;fapnum+=2)
    if(fapmask[frameind][fapnum] && (!fapmask[frameind][fapnum+1])) {
        lfaps[frameind][fapnum+1] = lfaps[frameind][fapnum];
        fapmask[frameind][fapnum+1] = 1;
    }
    else {
      if(fapmask[frameind][fapnum+1] && (!fapmask[frameind][fapnum])) {
          lfaps[frameind][fapnum] = lfaps[frameind][fapnum+1];
          fapmask[frameind][fapnum] = 1;
      }
    }
}

readfap(char *fapfilename, FDPtype *FDPs, FAPtype *FAPs) {
  FILE *fapfile;
  char cbuf[1000];
  int fapnum,frameind,i;
  int **isfeatpnt;
  float ***featpts;
  int **fapmask;
  int **faps;
  float **lfaps;
  int fap1[8000][4];
  int fap2[8000][6];

  isfeatpnt = FDPs->isfeatpnt;
  featpts = FDPs->featpts;
  fapmask = FAPs->fapmask;
  faps = FAPs->faps;
  lfaps = FAPs->lfaps;
  if(!(fapfile = fopen(fapfilename,"r"))) printf("ERROR: can't read %s\n",fapfilename);
  while(1) {/*read header line*/
    if(fscanf(fapfile,"%s",cbuf));
    else {
      printf("ERROR: bad fapfile: %s\n",cbuf);
      exit(2);
    }
    if(cbuf[0] == '#') while(getc(fapfile) != '\n');
    else {
      sscanf(cbuf,"%f ",&(FAPs->version));
      fscanf(fapfile,"%s %d %d",FAPs->basename,&(FAPs->framerate),&(FAPs->framecount));
      if(fabs(FAPs->version - 2.1) > 0.001)
	printf("WARNING: Expecting FAP file version 2.1 not %f\n",FAPs->version);
      while(getc(fapfile) != '\n');
      break;
    }
  }
  for(frameind=0;frameind<FAPs->framecount;frameind++) {
    while(1) {
      if(fscanf(fapfile,"%s",cbuf)!= EOF);
      else {
	printf("ERROR: bad fapfile: %s\n",cbuf);
	exit(2);
      }
      if(cbuf[0] == '#') while(getc(fapfile) != '\n');
      else {
	sscanf(cbuf,"%d",&fapmask[frameind][1]);
	for(fapnum=2;fapnum<69;fapnum++) fscanf(fapfile,"%d",&fapmask[frameind][fapnum]);
	while(getc(fapfile) != '\n');
	break;
      }
    }
    while(1) {
      if(fscanf(fapfile,"%s",cbuf)!= EOF);
      else {
	printf("ERROR: bad fapfile: %s\n",cbuf);
	exit(2);
      }
      if(cbuf[0] == '#') while(getc(fapfile) != '\n');
      else {
	sscanf(cbuf,"%d",&faps[frameind][0]);
	for(fapnum=1;fapnum<69;fapnum++) if(fapmask[frameind][fapnum]) {
	  if(fapnum == 1) {
	    fscanf(fapfile,"%d %d %d %d",&fap1[frameind][0],&fap1[frameind][1],&fap1[frameind][2],&fap1[frameind][3]);
	  }
	  else if(fapnum == 2) {
	    fscanf(fapfile,"%d %d %d %d %d %d",&fap2[frameind][0],&fap2[frameind][1],&fap2[frameind][2],&fap2[frameind][3],&fap2[frameind][4],&fap2[frameind][5]);
	  }
	  else fscanf(fapfile,"%d",&faps[frameind][fapnum]);
	}
	while(getc(fapfile) != '\n');
	break;
      }
    }
    for(fapnum=0;fapnum<68;fapnum++) lfaps[frameind][fapnum] = 0;
    for(fapnum=3;fapnum<6;fapnum++) if(fapmask[frameind][fapnum]) lfaps[frameind][fapnum] = ((float)faps[frameind][fapnum])/FDPs->MNSFAPUperpel;
    for(fapnum=6;fapnum<8;fapnum++) if(fapmask[frameind][fapnum]) lfaps[frameind][fapnum] = ((float)faps[frameind][fapnum])/FDPs->MNSFAPUperpel;
    for(fapnum=8;fapnum<19;fapnum++) if(fapmask[frameind][fapnum]) lfaps[frameind][fapnum] = ((float)faps[frameind][fapnum])/FDPs->MNSFAPUperpel;
    for(fapnum=19;fapnum<23;fapnum++) if(fapmask[frameind][fapnum]) lfaps[frameind][fapnum] = ((float)faps[frameind][fapnum])/FDPs->IRISDFAPUperpel;
    for(fapnum=23;fapnum<27;fapnum++) if(fapmask[frameind][fapnum]) lfaps[frameind][fapnum] = ((float)faps[frameind][fapnum])/100000.0;/*convert to radians*/
    for(fapnum=27;fapnum<29;fapnum++) if(fapmask[frameind][fapnum]) lfaps[frameind][fapnum] = ((float)faps[frameind][fapnum])/FDPs->ENSFAPUperpel;
    for(fapnum=29;fapnum<31;fapnum++)
      if(fapmask[frameind][fapnum]) lfaps[frameind][fapnum] = (((float)faps[frameind][fapnum])/FDPs->IRISDFAPUperpel)+(fabs(featpts[3][2][1] - featpts[3][4][1])/3.0);
      else lfaps[frameind][fapnum] = fabs(featpts[3][2][1] - featpts[3][4][1])/3.0;/*set pupil diameter to 1/3rd iris diameter;*/
    for(fapnum=31;fapnum<37;fapnum++) if(fapmask[frameind][fapnum]) lfaps[frameind][fapnum] = ((float)faps[frameind][fapnum])/FDPs->ENSFAPUperpel;
    for(fapnum=37;fapnum<41;fapnum++) if(fapmask[frameind][fapnum]) lfaps[frameind][fapnum] = ((float)faps[frameind][fapnum])/FDPs->ESFAPUperpel;
    for(fapnum=41;fapnum<43;fapnum++) if(fapmask[frameind][fapnum]) lfaps[frameind][fapnum] = ((float)faps[frameind][fapnum])/FDPs->ENSFAPUperpel;
    fapnum = 43;
    if(fapmask[frameind][fapnum]) lfaps[frameind][fapnum] = ((float)faps[frameind][fapnum])/FDPs->MWFAPUperpel;
    fapnum = 44;
    if(fapmask[frameind][fapnum]) lfaps[frameind][fapnum] = ((float)faps[frameind][fapnum])/FDPs->MNSFAPUperpel;
    fapnum = 45;
    if(fapmask[frameind][fapnum]) lfaps[frameind][fapnum] = ((float)faps[frameind][fapnum])/FDPs->MWFAPUperpel;
    fapnum = 46;
    if(fapmask[frameind][fapnum]) lfaps[frameind][fapnum] = ((float)faps[frameind][fapnum])/FDPs->MNSFAPUperpel;
    for(fapnum=47;fapnum<51;fapnum++) if(fapmask[frameind][fapnum]) lfaps[frameind][fapnum] = ((float)faps[frameind][fapnum])/100000.0;/*convert to radians*/
    for(fapnum=51;fapnum<53;fapnum++) if(fapmask[frameind][fapnum]) lfaps[frameind][fapnum] = ((float)faps[frameind][fapnum])/FDPs->MNSFAPUperpel;
    for(fapnum=53;fapnum<55;fapnum++) if(fapmask[frameind][fapnum]) lfaps[frameind][fapnum] = ((float)faps[frameind][fapnum])/FDPs->MWFAPUperpel;
    for(fapnum=55;fapnum<61;fapnum++) if(fapmask[frameind][fapnum]) lfaps[frameind][fapnum] = ((float)faps[frameind][fapnum])/FDPs->MNSFAPUperpel;
    for(fapnum=61;fapnum<68;fapnum++) if(fapmask[frameind][fapnum]) lfaps[frameind][fapnum] = ((float)faps[frameind][fapnum])/FDPs->ENSFAPUperpel;
    leftrightinterpolation(FAPs,frameind, 6, 12);/*inner lips*/
    leftrightinterpolation(FAPs,frameind, 53, 61);/*outer lips,nose*/
    leftrightinterpolation(FAPs,frameind, 19, 41);/*eyelids,eyebrows,eyeballs,pupils,cheeks*/
    leftrightinterpolation(FAPs,frameind, 65, 67);/*ears*/
  }
}


