/*
COPYRIGHT STATEMENT

NOTE: This copyright statement applies to all individual files or
modules in which the developers listed below are mentioned.

This software module was originally developed by the Lucent Technologies
Murray Hill, New Jersey, USA. Contributors to the software
include but are not limited to: Eric Petajan and Jialin Zhong.
This software module is an implementation of a part of one or more
MPEG-4 Visual (ISO/IEC 14496-2) tools as specified by MPEG-4 Visual
(ISO/IEC 14496-2). ISO/IEC gives users of the MPEG-4 Visual standard
free license to this software module or modifications thereof for
use in hardware or software products claiming conformance to the
MPEG-4 Visual (ISO/IEC 14496-2) standard. Those intending to use
this software module in hardware or software products are advised
that its use may infringe existing patents. The original developers
of this software and his/her company, the subsequent editors and
their companies, and ISO/IEC have no liability for use of this
software module or modifications thereof. Copyright is not released
for non MPEG-4 Visual standard conforming products. The original
developers retain full right to use the code for its own purpose,
assign or donate the code for non MPEG-4 Visual conforming products.
This copyright notice must be included in all copies or derivative
works.

Copyright (c) 1997

CONTRIBUTION TO THE WG11 PROGRAM MPEG 4 PLAYER
______________________________________________________________________

STANDARDS PROJECT:  MPEG 4 Player
______________________________________________________________________

TITLE:  3D Line-Based Face Animation Module
______________________________________________________________________

AUTHORS:  Eric Petajan, Jialin Zhong

SOURCE:      Lucent Technologies Inc.

CONTACT:        Eric Petajan
                Lucent Technologies
                Room 2B231
                600 Mt. Ave.
                Murray Hill, NJ  07974
                Voice:  +1 908-582-3160
                Fax: +1 908-582-6632
                Email:  edp@bell-labs.com
______________________________________________________________________


ABSTRACT:  this module inputs face animation and definition parameters,
and outputs 3D points, lines, and polygons.
*/


#include <stdio.h>
#include <stddef.h>
#include <stdlib.h>
#define NR_END 1
#define FREE_ARG char*

float **matrix(long nrl, long nrh, long ncl, long nch)
/* allocate a float matrix with subscript range m[nrl..nrh][ncl..nch] */
{
	long i, nrow=nrh-nrl+1,ncol=nch-ncl+1;
	float **m;

	/* allocate pointers to rows */
	m=(float **) malloc((size_t)((nrow+NR_END)*sizeof(float*)));
	if (!m) printf("allocation failure 1 in matrix()");
	m += NR_END;
	m -= nrl;

	/* allocate rows and set pointers to them */
	m[nrl]=(float *) malloc((size_t)((nrow*ncol+NR_END)*sizeof(float)));
	if (!m[nrl]) printf("allocation failure 2 in matrix()");
	m[nrl] += NR_END;
	m[nrl] -= ncl;

	for(i=nrl+1;i<=nrh;i++) m[i]=m[i-1]+ncol;

	/* return pointer to array of pointers to rows */
	return m;
}

void free_matrix(float **m, long nrl, long nrh, long ncl, long nch)
/* free a float matrix allocated by matrix() */
{
	free((FREE_ARG) (m[nrl]+ncl-NR_END));
	free((FREE_ARG) (m+nrl-NR_END));
}
