/*
COPYRIGHT STATEMENT

NOTE: This copyright statement applies to all individual files or
modules in which the developers listed below are mentioned.

This software module was originally developed by the Lucent Technologies
Murray Hill, New Jersey, USA. Contributors to the software
include but are not limited to: Eric Petajan and Jialin Zhong.
This software module is an implementation of a part of one or more
MPEG-4 Visual (ISO/IEC 14496-2) tools as specified by MPEG-4 Visual
(ISO/IEC 14496-2). ISO/IEC gives users of the MPEG-4 Visual standard
free license to this software module or modifications thereof for
use in hardware or software products claiming conformance to the
MPEG-4 Visual (ISO/IEC 14496-2) standard. Those intending to use
this software module in hardware or software products are advised
that its use may infringe existing patents. The original developers
of this software and his/her company, the subsequent editors and
their companies, and ISO/IEC have no liability for use of this
software module or modifications thereof. Copyright is not released
for non MPEG-4 Visual standard conforming products. The original
developers retain full right to use the code for its own purpose,
assign or donate the code for non MPEG-4 Visual conforming products.
This copyright notice must be included in all copies or derivative
works.

Copyright (c) 1997

CONTRIBUTION TO THE WG11 PROGRAM MPEG 4 PLAYER
______________________________________________________________________

STANDARDS PROJECT:  MPEG 4 Player
______________________________________________________________________

TITLE:  3D Line-Based Face Animation Module
______________________________________________________________________

AUTHORS:  Eric Petajan, Jialin Zhong

SOURCE:      Lucent Technologies Inc.

CONTACT:        Eric Petajan
                Lucent Technologies
                Room 2B231
                600 Mt. Ave.
                Murray Hill, NJ  07974
                Voice:  +1 908-582-3160
                Fax: +1 908-582-6632
                Email:  edp@bell-labs.com
______________________________________________________________________


ABSTRACT:  this module inputs face animation and definition parameters,
and outputs 3D points, lines, and polygons.
*/

#include <sys/time.h>
#include "wireface.h"

float coord[2][3]={
  {150,220,220},
  {-150,220,220}
};

static int dblBuf[] = {
  GLX_DOUBLEBUFFER,
  GLX_RGBA,
  GLX_DEPTH_SIZE, 16,
  GLX_RED_SIZE, 1,
  GLX_GREEN_SIZE, 1,
  GLX_BLUE_SIZE, 1,
  None };
static int *snglBuf = &dblBuf[1];
static String   fallbackResources[] = {
  "*sgiMode: true",           /* try to enable IRIX 5.2+ look & feel */
  "*useSchemes: all",         /* and SGI schemes */
  "*title: wireface",
  "*glxarea*foreground: blue2",
  "*glxarea*background: blue2",
  "*glxarea*width: 640", "*glxarea*height: 480", NULL
};
Display	*dpy;
XtAppContext	app;
XVisualInfo	*vi;
XtWorkProcId	workId = 0;

GLXContext	cx;

Widget glxarea;

GLboolean doubleBuffer=GL_TRUE; 
GLboolean made_current=GL_FALSE, moving=GL_FALSE;
GLboolean start=GL_TRUE;
GLboolean shadedMode=GL_TRUE, earMode=GL_FALSE, wireMode=GL_FALSE, textureMode=GL_FALSE;
GLboolean TextureFirst=GL_FALSE;

GLint Image_Width=IMAGE_WIDTH, Image_Height=IMAGE_HEIGHT;

GLuint InitList; 

Widget toplevel;

int framenum = 0,todi=5,isaudio=0;
struct timeval tod[5];
double toddiff[5],toterr=0.0,peakerror=0.0;
FDPtype FDPs;
FAPtype FAPs;
int frate,maxframes;

main(int argc, char *argv[]) {
  char *UserName,cbuf[256],cdummy[256];
  float version;
  int i,j, c;
  Widget mainw,frame;
  FILE *fapfile;


  if((argc != 3) && (argc != 4)) {
    printf("Usage: wireface <fdp filename> <fap filename> <audio file - raw 16bit linear stereo 48KHz>\n Version 2.0\n");
    exit(2);
  }
  if(!(fapfile = fopen(argv[2],"r"))) printf("ERROR: can't read %s\n",argv[2]);
  while(1) {/*just read header line for mallocs below*/
    if(fscanf(fapfile,"%s",cbuf));
    else {
      printf("ERROR: bad fapfile: %s\n",cbuf);
      exit(2);
    }
    if(cbuf[0] == '#') while(getc(fapfile) != '\n');
    else {
      sscanf(cbuf,"%f ",&version);
      fscanf(fapfile,"%s %d %d",cdummy,&frate,&maxframes);
      if(fabs(version - 2.1) > 0.001)
	printf("WARNING: Expecting FAP file version 2.1 not %f\n",version);
      while(getc(fapfile) != '\n');
      break;
    }
  }
  fclose(fapfile);
  if(argc == 4) {
    init_audio((float)(maxframes)/(float)frate,argv[3]);
    isaudio = 1;
  }
  if(!(FDPs.isfeatpnt = (int **)malloc(16*sizeof(int *)))) {
    printf("ERROR: malloc failed on FDPs.isfeatpnt\n");
    exit(2);
  }
  for(i=0;i<16;i++)
    if(!(FDPs.isfeatpnt[i] = (int *)malloc(20*sizeof(int )))) {
      printf("ERROR: malloc failed on FDPs.isfeatpnt\n");
      exit(2);
    }
  if(!(FDPs.featpts = (float ***)malloc(16*sizeof(float **)))) {
    printf("ERROR: malloc failed on FDPs.isfeatpnt\n");
    exit(2);
  }
  for(i=0;i<16;i++) {
    if(!(FDPs.featpts[i] = (float **)malloc(20*sizeof(float *)))) {
      printf("ERROR: malloc failed on FDPs.featpts\n");
      exit(2);
    }
    for(j=0;j<20;j++) if(!(FDPs.featpts[i][j] = (float *)malloc(5*sizeof(float)))) {
      printf("ERROR: malloc failed on FDPs.featpts\n");
      exit(2);
    }
  }

  if(!(FAPs.fapmask = (int **)malloc(maxframes*sizeof(int *)))) {
    printf("ERROR: malloc failed on FAPs.fapmask\n");
    exit(2);
  }
  for(i=0;i<maxframes;i++)
    if(!(FAPs.fapmask[i] = (int *)malloc(69*sizeof(int )))) {
      printf("ERROR: malloc failed on FAPs.fapmask\n");
      exit(2);
    }
  if(!(FAPs.faps = (int **)malloc(maxframes*sizeof(int *)))) {
    printf("ERROR: malloc failed on FAPs.faps\n");
    exit(2);
  }
  for(i=0;i<maxframes;i++)
    if(!(FAPs.faps[i] = (int *)malloc(69*sizeof(int )))) {
      printf("ERROR: malloc failed on FAPs.faps\n");
      exit(2);
    }
  if(!(FAPs.lfaps = (float **)malloc(maxframes*sizeof(float *)))) {
    printf("ERROR: malloc failed on FAPs.lfaps\n");
    exit(2);
  }
  for(i=0;i<maxframes;i++)
    if(!(FAPs.lfaps[i] = (float *)malloc(69*sizeof(float )))) {
      printf("ERROR: malloc failed on FAPs.lfaps\n");
      exit(2);
    }
  if(!(FAPs.basename = (char *)malloc(256*sizeof(char )))) {
    printf("ERROR: malloc failed on FAPs.basename\n");
    exit(2);
  }

  readfdp(argv[1],&FDPs);
  readfap(argv[2],&FDPs,&FAPs);

  UserName=getenv("LOGNAME");
  toplevel=XtAppInitialize(&app,"wireface",NULL,0,&argc,argv, fallbackResources,NULL,0);
  dpy=XtDisplay(toplevel);
  vi=glXChooseVisual(dpy, DefaultScreen(dpy), dblBuf);
  if(vi == NULL) {
      vi=glXChooseVisual(dpy,DefaultScreen(dpy),snglBuf);
      printf("single buffer display used\n");
      if(vi == NULL)
         XtAppError(app,"no RGB visual with depth buffer");
      doubleBuffer=GL_FALSE;
      }
   if((cx=glXCreateContext(dpy,vi,None,GL_TRUE)) == NULL)
      XtAppError(app,"could not create rendering context");
   XtAddEventHandler(toplevel,StructureNotifyMask,False, 
		map_state_changed,NULL);

/*create the main window*/
   mainw=XmCreateMainWindow(toplevel, "mainw", NULL, 0);
   XtManageChild(mainw);
   frame=XmCreateFrame(mainw,"frame",NULL,0);
   XtManageChild(frame);
   glxarea=XtVaCreateManagedWidget("glxarea",glwMDrawingAreaWidgetClass,
		frame,GLwNvisualInfo,vi,NULL);
/*standard event callbacks*/
   XtAddCallback(glxarea,XmNexposeCallback,(XtCallbackProc)plot,NULL);
   XtAddCallback(glxarea,XmNresizeCallback,(XtCallbackProc)resize,NULL);
   XtRealizeWidget(toplevel);
/*binding opengl to window*/
   if((made_current=glXMakeCurrent(dpy,XtWindow(glxarea),cx))==GL_FALSE)
      XtAppError(app,"can't bind opengl to window");
/*input from stdin*/
   /*    XtAppAddInput(app, fileno(stdin), XtInputReadMask, getFile, NULL);*/
   XtAppMainLoop(app);
}

void map_state_changed(Widget w, XtPointer data, XEvent * event, Boolean * cont)
{
   switch (event->type) {
   case MapNotify:
      if(moving && workId != 0) workId=XtAppAddWorkProc(app,(XtWorkProc)plot,NULL);
      break;
   case UnmapNotify:
      if (moving) XtRemoveWorkProc(workId);
      break;
   }
}

void resize(Widget w, XtPointer data, XtPointer callData) {
   Dimension       width, height;

   if(made_current) {
      XtVaGetValues(w, XmNwidth, &width, XmNheight, &height, NULL);
      glViewport(-50, -50, (GLint) width, (GLint) height);
      Image_Width=(GLint) width; Image_Height=(GLint)height;
   }
}


void plot(Widget w) {
  int i,toderri1,toderri2,tooslow=0;
  float **crd;
  double t1, t2, t3,toderr;
  float leyerot,reyerot,headrot;
  FILE *fp;
  while(1) {  
    crd=(float **)matrix(0,1,0,2);
    for(i=0;i<2;i++){
      crd[i][0]=coord[i][0];
      crd[i][1]=coord[i][1];
      crd[i][2]=coord[i][2]; }
    plot_init(glxarea);
    /*uncomment this section to test head and eye rotations    
    if(!fapmask[framenum][25]) {
      leyerot = 0;
      reyerot = 0;
      headrot = 0;
      fapmask[framenum][25] = 1;
      fapmask[framenum][23] = 1;
      fapmask[framenum][24] = 1;
      fapmask[framenum][26] = 1;
      fapmask[framenum][48] = 1;
      fapmask[framenum][49] = 1;
      fapmask[framenum][50] = 1;
    }
    lfaps[framenum][25] = leyerot;
    lfaps[framenum][23] = leyerot;
    lfaps[framenum][24] = reyerot;
    lfaps[framenum][26] = reyerot;
    lfaps[framenum][48] = headrot;
    lfaps[framenum][49] = headrot;
    lfaps[framenum][50] = headrot;
    leyerot += 0.01;
    reyerot += 0.02;
    headrot += 0.005;
    if(leyerot >= 2*M_PI) {
      leyerot = 0;
    }
    if(reyerot >= 2*M_PI) {
      reyerot = 0;
    }
    if(headrot >= 2*M_PI) {
      headrot = 0;
    }
    */
    if(todi == 5) {/*init*/
      for(i=0;i<5;i++) tod[i].tv_usec = -1;
      for(i=0;i<5;i++)toddiff[i] = 0;
      todi = framenum%4;
      gettimeofday(&(tod[todi]), (struct timezone *)0);
    }
    else {
      todi = framenum%4;
      gettimeofday(&(tod[todi]), (struct timezone *)0);
      if((framenum > 0) && (tod[(framenum)%4].tv_usec != -1) && (tod[(framenum-1)%4].tv_usec != -1)) {
	toderri1 = (framenum)%4;
	toderri2 = (framenum-1)%4;
      }
      else {
	toderri1 = todi;
	toderri2 = todi;
	}
      if(tod[toderri1].tv_usec -tod[toderri2].tv_usec < 0)
	toderr = ((double)(tod[toderri1].tv_usec -tod[toderri2].tv_usec + 1000000)/1000000.0) - (1.0/((double)FAPs.framerate));
      else
	toderr = ((double)(tod[toderri1].tv_usec -tod[toderri2].tv_usec)/1000000.0) - (1.0/((double)FAPs.framerate));
      if(toderr > 1.0/((double)FAPs.framerate))
	tooslow = 1;
      toterr += toderr;
      if(toderr > peakerror) peakerror = toderr;
      if(peakerror > 0.025) {
	peakerror += 0;
      }
	
      do {
	gettimeofday(&(tod[4]), (struct timezone *)0);
	if(tod[4].tv_usec - tod[todi].tv_usec < 0) toddiff[todi] = ((double)(tod[4].tv_usec - tod[todi].tv_usec + 1000000)/1000000.0);
	else toddiff[todi] = ((double)(tod[4].tv_usec - tod[todi].tv_usec)/1000000.0);
      } while(toddiff[todi]  < toddiff[toderri2] - toderr);
    }
    applyfaps(framenum++,&FDPs,&FAPs);
    if((framenum == 1) && isaudio) {/*play audio*/
      play_audio();
    }
    if(framenum >= FAPs.framecount) {
      /*printf("At beginning of sequence time = %d  toddiff = %f toderr = %f\n",tod[4].tv_sec,toddiff[todi], toderr);*/
      framenum=0;
      todi = 5;
      if(tooslow) printf("WARNING: this machine is too slow for realtime animation\n");
      printf("Average time error per frame = %f  peak time error = %f tot error = %f\n",toterr/FAPs.framecount,peakerror,toterr);
      peakerror = 0.0;
      tooslow = 0;
      toterr = 0.0;
    }
    free_matrix(crd,0,1,0,2);
    if(doubleBuffer) glXSwapBuffers(dpy, XtWindow(w));
    if(!glXIsDirect(dpy, cx)) 
      glFlush();
  }
}



